@@ -103,7 +103,7 @@ export EXCHANGE_URL=`taler-config -c $CONF -s EXCHANGE -o BASE_URL`
 MERCHANT_PORT=`taler-config -c $CONF -s MERCHANT -o PORT`
 export MERCHANT_URL=http://localhost:${MERCHANT_PORT}/
 BANK_PORT=`taler-config -c $CONF -s BANK -o HTTP_PORT`
-export BANK_URL=http://localhost:1${BANK_PORT}/demobanks/default
+export BANK_URL=http://localhost:1${BANK_PORT}
 export AUDITOR_URL=http://localhost:8083/
 AUDITOR_PRIV_FILE=$1.apriv
 AUDITOR_PRIV_DIR=`dirname $AUDITOR_PRIV_FILE`
@@ -152,7 +152,7 @@ libeufin-sandbox serve --port "1${BANK_PORT}" \
   2> ${MY_TMP_DIR}/libeufin-sandbox-stderr.log &
 echo $! > ${MY_TMP_DIR}/libeufin-sandbox.pid
 cd $ORIGIN
-export LIBEUFIN_SANDBOX_URL="http://localhost:1${BANK_PORT}/demobanks/default"
+export LIBEUFIN_SANDBOX_URL="http://localhost:1${BANK_PORT}"
 set +e
 echo -n "Waiting for Sandbox..."
 OK=0
@@ -413,7 +413,7 @@ taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api --expect-success 'with
       bankBaseUrl: $BANK_URL,
       exchangeBaseUrl: $EXCHANGE_URL,
     }' \
-    --arg BANK_URL "$BANK_URL/access-api/" \
+    --arg BANK_URL "$BANK_URL/demobanks/default/access-api/" \
     --arg EXCHANGE_URL $EXCHANGE_URL
   )"
 
