@@ -21,6 +21,7 @@
  */
 #include "platform.h"
 #include <gnunet/gnunet_util_lib.h>
+#include "taler_util.h"
 #include "taler_auditordb_plugin.h"
 
 
@@ -69,7 +70,7 @@ run (void *cls,
   {
     fprintf (stderr,
              "Failed to initialize database plugin.\n");
-    global_ret = 1;
+    global_ret = EXIT_NOTINSTALLED;
     return;
   }
   if (reset_db)
@@ -94,7 +95,7 @@ run (void *cls,
     fprintf (stderr,
              "Failed to initialize database.\n");
     TALER_AUDITORDB_plugin_unload (plugin);
-    global_ret = 1;
+    global_ret = EXIT_NOPERMISSION;
     return;
   }
   if (gc_db)
@@ -134,22 +135,27 @@ main (int argc,
                                &gc_db),
     GNUNET_GETOPT_OPTION_END
   };
+  enum GNUNET_GenericReturnValue ret;
 
+  if (GNUNET_OK !=
+      GNUNET_STRINGS_get_utf8_args (argc, argv,
+                                    &argc, &argv))
+    return EXIT_INVALIDARGUMENT;
   /* force linker to link against libtalerutil; if we do
      not do this, the linker may "optimize" libtalerutil
      away and skip #TALER_OS_init(), which we do need */
-  (void) TALER_project_data_default ();
-  GNUNET_assert (GNUNET_OK ==
-                 GNUNET_log_setup ("taler-auditor-dbinit",
-                                   "INFO",
-                                   NULL));
-  if (GNUNET_OK !=
-      GNUNET_PROGRAM_run (argc, argv,
-                          "taler-auditor-dbinit",
-                          "Initialize Taler auditor database",
-                          options,
-                          &run, NULL))
-    return 1;
+  TALER_OS_init ();
+  ret = GNUNET_PROGRAM_run (
+    argc, argv,
+    "taler-auditor-dbinit",
+    gettext_noop ("Initialize Taler auditor database"),
+    options,
+    &run, NULL);
+  GNUNET_free_nz ((void *) argv);
+  if (GNUNET_SYSERR == ret)
+    return EXIT_INVALIDARGUMENT;
+  if (GNUNET_NO == ret)
+    return EXIT_SUCCESS;
   return global_ret;
 }
 
