@@ -32,10 +32,54 @@
 #include "taler-auditor-httpd_deposit-confirmation.h"
 
 
+GNUNET_NETWORK_STRUCT_BEGIN
+
+/**
+ * @brief Information about a signing key of the exchange.  Signing keys are used
+ * to sign exchange messages other than coins, i.e. to confirm that a
+ * deposit was successful or that a refresh was accepted.
+ */
+struct ExchangeSigningKeyDataP
+{
+
+  /**
+   * When does this signing key begin to be valid?
+   */
+  struct GNUNET_TIME_TimestampNBO start;
+
+  /**
+   * When does this signing key expire? Note: This is currently when
+   * the Exchange will definitively stop using it.  Signatures made with
+   * the key remain valid until @e end.  When checking validity periods,
+   * clients should allow for some overlap between keys and tolerate
+   * the use of either key during the overlap time (due to the
+   * possibility of clock skew).
+   */
+  struct GNUNET_TIME_TimestampNBO expire;
+
+  /**
+   * When do signatures with this signing key become invalid?  After
+   * this point, these signatures cannot be used in (legal) disputes
+   * anymore, as the Exchange is then allowed to destroy its side of the
+   * evidence.  @e end is expected to be significantly larger than @e
+   * expire (by a year or more).
+   */
+  struct GNUNET_TIME_TimestampNBO end;
+
+  /**
+   * The public online signing key that the exchange will use
+   * between @e start and @e expire.
+   */
+  struct TALER_ExchangePublicKeyP signkey_pub;
+};
+
+GNUNET_NETWORK_STRUCT_END
+
+
 /**
  * Cache of already verified exchange signing keys.  Maps the hash of the
  * `struct TALER_ExchangeSigningKeyValidityPS` to the (static) string
- * "verified".  Access to this map is guarded by the #lock.
+ * "verified" or "revoked".  Access to this map is guarded by the #lock.
  */
 static struct GNUNET_CONTAINER_MultiHashMap *cache;
 
@@ -62,24 +106,18 @@ verify_and_execute_deposit_confirmation (
   const struct TALER_AUDITORDB_DepositConfirmation *dc,
   const struct TALER_AUDITORDB_ExchangeSigningKey *es)
 {
-  struct TALER_AUDITORDB_Session *session;
   enum GNUNET_DB_QueryStatus qs;
-  struct GNUNET_TIME_Absolute now;
   struct GNUNET_HashCode h;
-  int cached;
-  struct TALER_ExchangeSigningKeyValidityPS skv = {
-    .purpose.purpose = htonl (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY),
-    .purpose.size = htonl (sizeof (struct TALER_ExchangeSigningKeyValidityPS)),
-    .master_public_key = es->master_public_key,
-    .start = GNUNET_TIME_absolute_hton (es->ep_start),
-    .expire = GNUNET_TIME_absolute_hton (es->ep_expire),
-    .end = GNUNET_TIME_absolute_hton (es->ep_end),
+  const char *cached;
+  struct ExchangeSigningKeyDataP skv = {
+    .start = GNUNET_TIME_timestamp_hton (es->ep_start),
+    .expire = GNUNET_TIME_timestamp_hton (es->ep_expire),
+    .end = GNUNET_TIME_timestamp_hton (es->ep_end),
     .signkey_pub = es->exchange_pub
   };
 
-  now = GNUNET_TIME_absolute_get ();
-  if ( (es->ep_start.abs_value_us > now.abs_value_us) ||
-       (es->ep_expire.abs_value_us < now.abs_value_us) )
+  if (GNUNET_TIME_absolute_is_future (es->ep_start.abs_time) ||
+      GNUNET_TIME_absolute_is_past (es->ep_expire.abs_time) )
   {
     /* Signing key expired */
     TALER_LOG_WARNING ("Expired exchange signing key\n");
@@ -94,12 +132,11 @@ verify_and_execute_deposit_confirmation (
                       sizeof (skv),
                       &h);
   GNUNET_assert (0 == pthread_mutex_lock (&lock));
-  cached = GNUNET_CONTAINER_multihashmap_contains (cache,
-                                                   &h);
+  cached = GNUNET_CONTAINER_multihashmap_get (cache,
+                                              &h);
   GNUNET_assert (0 == pthread_mutex_unlock (&lock));
-
-  session = TAH_plugin->get_session (TAH_plugin->cls);
-  if (NULL == session)
+  if (GNUNET_SYSERR ==
+      TAH_plugin->preflight (TAH_plugin->cls))
   {
     GNUNET_break (0);
     return TALER_MHD_reply_with_error (connection,
@@ -107,14 +144,17 @@ verify_and_execute_deposit_confirmation (
                                        TALER_EC_GENERIC_DB_SETUP_FAILED,
                                        NULL);
   }
-  if (! cached)
+  if (NULL == cached)
   {
     /* Not in cache, need to verify the signature, persist it, and possibly cache it */
     if (GNUNET_OK !=
-        GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY,
-                                    &skv,
-                                    &es->master_sig.eddsa_signature,
-                                    &es->master_public_key.eddsa_pub))
+        TALER_exchange_offline_signkey_validity_verify (
+          &es->exchange_pub,
+          es->ep_start,
+          es->ep_expire,
+          es->ep_end,
+          &es->master_public_key,
+          &es->master_sig))
     {
       TALER_LOG_WARNING ("Invalid signature on exchange signing key\n");
       return TALER_MHD_reply_with_error (connection,
@@ -125,7 +165,6 @@ verify_and_execute_deposit_confirmation (
 
     /* execute transaction */
     qs = TAH_plugin->insert_exchange_signkey (TAH_plugin->cls,
-                                              session,
                                               es);
     if (0 > qs)
     {
@@ -136,51 +175,78 @@ verify_and_execute_deposit_confirmation (
                                          TALER_EC_GENERIC_DB_STORE_FAILED,
                                          "exchange signing key");
     }
+    cached = "verified";
+  }
 
-    /* Cache it, due to concurreny it might already be in the cache,
-       so we do not cache it twice but also don't insist on the 'put' to
-       succeed. */
-    GNUNET_assert (0 == pthread_mutex_lock (&lock));
-    (void) GNUNET_CONTAINER_multihashmap_put (cache,
-                                              &h,
-                                              "verified",
-                                              GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY);
-    GNUNET_assert (0 == pthread_mutex_unlock (&lock));
+  if (0 == strcmp (cached,
+                   "verified"))
+  {
+    struct TALER_MasterSignatureP master_sig;
+
+    /* check for revocation */
+    qs = TAH_eplugin->lookup_signkey_revocation (TAH_eplugin->cls,
+                                                 &es->exchange_pub,
+                                                 &master_sig);
+    if (0 > qs)
+    {
+      GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
+      TALER_LOG_WARNING (
+        "Failed to check for signing key revocation in database\n");
+      return TALER_MHD_reply_with_error (connection,
+                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                         TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                         "exchange signing key revocation");
+    }
+    if (0 < qs)
+      cached = "revoked";
+  }
+
+  /* Cache it, due to concurreny it might already be in the cache,
+     so we do not cache it twice but also don't insist on the 'put' to
+     succeed. */
+  GNUNET_assert (0 == pthread_mutex_lock (&lock));
+  (void) GNUNET_CONTAINER_multihashmap_put (cache,
+                                            &h,
+                                            (void *) cached,
+                                            GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY);
+  GNUNET_assert (0 == pthread_mutex_unlock (&lock));
+
+  if (0 == strcmp (cached,
+                   "revoked"))
+  {
+    TALER_LOG_WARNING (
+      "Invalid signature on /deposit-confirmation request: key was revoked\n");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_GONE,
+                                       TALER_EC_AUDITOR_EXCHANGE_SIGNING_KEY_REVOKED,
+                                       "exchange signing key was revoked");
   }
 
   /* check deposit confirmation signature */
+  if (GNUNET_OK !=
+      TALER_exchange_online_deposit_confirmation_verify (
+        &dc->h_contract_terms,
+        &dc->h_wire,
+        &dc->h_policy,
+        dc->exchange_timestamp,
+        dc->wire_deadline,
+        dc->refund_deadline,
+        &dc->amount_without_fee,
+        &dc->coin_pub,
+        &dc->merchant,
+        &dc->exchange_pub,
+        &dc->exchange_sig))
   {
-    struct TALER_DepositConfirmationPS dcs = {
-      .purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT),
-      .purpose.size = htonl (sizeof (struct TALER_DepositConfirmationPS)),
-      .h_contract_terms = dc->h_contract_terms,
-      .h_wire = dc->h_wire,
-      .exchange_timestamp = GNUNET_TIME_absolute_hton (dc->exchange_timestamp),
-      .refund_deadline = GNUNET_TIME_absolute_hton (dc->refund_deadline),
-      .coin_pub = dc->coin_pub,
-      .merchant = dc->merchant
-    };
-
-    TALER_amount_hton (&dcs.amount_without_fee,
-                       &dc->amount_without_fee);
-    if (GNUNET_OK !=
-        GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT,
-                                    &dcs,
-                                    &dc->exchange_sig.eddsa_signature,
-                                    &dc->exchange_pub.eddsa_pub))
-    {
-      TALER_LOG_WARNING (
-        "Invalid signature on /deposit-confirmation request\n");
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_FORBIDDEN,
-                                         TALER_EC_AUDITOR_DEPOSIT_CONFIRMATION_SIGNATURE_INVALID,
-                                         "exchange signature invalid");
-    }
+    TALER_LOG_WARNING (
+      "Invalid signature on /deposit-confirmation request\n");
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_FORBIDDEN,
+                                       TALER_EC_AUDITOR_DEPOSIT_CONFIRMATION_SIGNATURE_INVALID,
+                                       "exchange signature invalid");
   }
 
   /* execute transaction */
   qs = TAH_plugin->insert_deposit_confirmation (TAH_plugin->cls,
-                                                session,
                                                 dc);
   if (0 > qs)
   {
@@ -191,27 +257,13 @@ verify_and_execute_deposit_confirmation (
                                        TALER_EC_GENERIC_DB_STORE_FAILED,
                                        "deposit confirmation");
   }
-  return TALER_MHD_reply_json_pack (connection,
+  return TALER_MHD_REPLY_JSON_PACK (connection,
                                     MHD_HTTP_OK,
-                                    "{s:s}",
-                                    "status", "DEPOSIT_CONFIRMATION_OK");
+                                    GNUNET_JSON_pack_string ("status",
+                                                             "DEPOSIT_CONFIRMATION_OK"));
 }
 
 
-/**
- * Handle a "/deposit-confirmation" request.  Parses the JSON, and, if
- * successful, passes the JSON data to #verify_and_execute_deposit_confirmation()
- * to further check the details of the operation specified.  If
- * everything checks out, this will ultimately lead to the "/deposit-confirmation"
- * being stored in the database.
- *
- * @param rh context of the handler
- * @param connection the MHD connection to handle
- * @param[in,out] connection_cls the connection's closure (can be updated)
- * @param upload_data upload data
- * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
- * @return MHD result code
-  */
 MHD_RESULT
 TAH_DEPOSIT_CONFIRMATION_handler (struct TAH_RequestHandler *rh,
                                   struct MHD_Connection *connection,
@@ -222,21 +274,39 @@ TAH_DEPOSIT_CONFIRMATION_handler (struct TAH_RequestHandler *rh,
   struct TALER_AUDITORDB_DepositConfirmation dc;
   struct TALER_AUDITORDB_ExchangeSigningKey es;
   struct GNUNET_JSON_Specification spec[] = {
-    GNUNET_JSON_spec_fixed_auto ("h_contract_terms", &dc.h_contract_terms),
-    GNUNET_JSON_spec_fixed_auto ("h_wire", &dc.h_wire),
-    TALER_JSON_spec_absolute_time ("exchange_timestamp",
-                                   &dc.exchange_timestamp),
-    TALER_JSON_spec_absolute_time ("refund_deadline", &dc.refund_deadline),
-    TALER_JSON_spec_amount ("amount_without_fee", &dc.amount_without_fee),
-    GNUNET_JSON_spec_fixed_auto ("coin_pub", &dc.coin_pub),
-    GNUNET_JSON_spec_fixed_auto ("merchant_pub", &dc.merchant),
-    GNUNET_JSON_spec_fixed_auto ("exchange_sig",  &dc.exchange_sig),
-    GNUNET_JSON_spec_fixed_auto ("exchange_pub",  &dc.exchange_pub),
-    GNUNET_JSON_spec_fixed_auto ("master_pub",  &es.master_public_key),
-    TALER_JSON_spec_absolute_time ("ep_start",  &es.ep_start),
-    TALER_JSON_spec_absolute_time ("ep_expire",  &es.ep_expire),
-    TALER_JSON_spec_absolute_time ("ep_end",  &es.ep_end),
-    GNUNET_JSON_spec_fixed_auto ("master_sig",  &es.master_sig),
+    GNUNET_JSON_spec_fixed_auto ("h_contract_terms",
+                                 &dc.h_contract_terms),
+    GNUNET_JSON_spec_fixed_auto ("h_policy",
+                                 &dc.h_policy),
+    GNUNET_JSON_spec_fixed_auto ("h_wire",
+                                 &dc.h_wire),
+    GNUNET_JSON_spec_timestamp ("exchange_timestamp",
+                                &dc.exchange_timestamp),
+    GNUNET_JSON_spec_timestamp ("refund_deadline",
+                                &dc.refund_deadline),
+    GNUNET_JSON_spec_timestamp ("wire_deadline",
+                                &dc.wire_deadline),
+    TALER_JSON_spec_amount ("amount_without_fee",
+                            TAH_currency,
+                            &dc.amount_without_fee),
+    GNUNET_JSON_spec_fixed_auto ("coin_pub",
+                                 &dc.coin_pub),
+    GNUNET_JSON_spec_fixed_auto ("merchant_pub",
+                                 &dc.merchant),
+    GNUNET_JSON_spec_fixed_auto ("exchange_sig",
+                                 &dc.exchange_sig),
+    GNUNET_JSON_spec_fixed_auto ("exchange_pub",
+                                 &dc.exchange_pub),
+    GNUNET_JSON_spec_fixed_auto ("master_pub",
+                                 &es.master_public_key),
+    GNUNET_JSON_spec_timestamp ("ep_start",
+                                &es.ep_start),
+    GNUNET_JSON_spec_timestamp ("ep_expire",
+                                &es.ep_expire),
+    GNUNET_JSON_spec_timestamp ("ep_end",
+                                &es.ep_end),
+    GNUNET_JSON_spec_fixed_auto ("master_sig",
+                                 &es.master_sig),
     GNUNET_JSON_spec_end ()
   };
 
@@ -282,9 +352,6 @@ TAH_DEPOSIT_CONFIRMATION_handler (struct TAH_RequestHandler *rh,
 }
 
 
-/**
- * Initialize subsystem.
- */
 void
 TEAH_DEPOSIT_CONFIRMATION_init (void)
 {
@@ -294,15 +361,15 @@ TEAH_DEPOSIT_CONFIRMATION_init (void)
 }
 
 
-/**
- * Shut down subsystem.
- */
 void
 TEAH_DEPOSIT_CONFIRMATION_done (void)
 {
-  GNUNET_CONTAINER_multihashmap_destroy (cache);
-  cache = NULL;
-  GNUNET_assert (0 == pthread_mutex_destroy (&lock));
+  if (NULL != cache)
+  {
+    GNUNET_CONTAINER_multihashmap_destroy (cache);
+    cache = NULL;
+    GNUNET_assert (0 == pthread_mutex_destroy (&lock));
+  }
 }
 
 
