@@ -36,7 +36,7 @@
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
  * @return MHD result code
   */
-int
+MHD_RESULT
 TAH_EXCHANGES_handler (struct TAH_RequestHandler *rh,
                        struct MHD_Connection *connection,
                        void **connection_cls,
