@@ -43,7 +43,7 @@
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
  * @return MHD result code
  */
-int
+MHD_RESULT
 TAH_MHD_handler_static_response (struct TAH_RequestHandler *rh,
                                  struct MHD_Connection *connection,
                                  void **connection_cls,
@@ -77,7 +77,7 @@ TAH_MHD_handler_static_response (struct TAH_RequestHandler *rh,
  * @param[in,out] upload_data_size number of bytes (left) in @a upload_data
  * @return MHD result code
  */
-int
+MHD_RESULT
 TAH_MHD_handler_agpl_redirect (struct TAH_RequestHandler *rh,
                                struct MHD_Connection *connection,
                                void **connection_cls,
