@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2016-2020 Taler Systems SA
+  Copyright (C) 2016-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero Public License as published by the Free Software
@@ -51,6 +51,10 @@ static json_int_t number_missed_deposit_confirmations;
  */
 static struct TALER_Amount total_missed_deposit_confirmations;
 
+/**
+ * Should we run checks that only work for exchange-internal audits?
+ */
+static int internal_checks;
 
 /**
  * Closure for #test_dc.
@@ -98,7 +102,7 @@ struct DepositConfirmationContext
  * @param dc the deposit confirmation we know
  * @return #GNUNET_OK to continue to iterate, #GNUNET_SYSERR to stop iterating
  */
-static int
+static enum GNUNET_GenericReturnValue
 test_dc (void *cls,
          uint64_t serial_id,
          const struct TALER_AUDITORDB_DepositConfirmation *dc)
@@ -108,27 +112,22 @@ test_dc (void *cls,
   dcc->last_seen_coin_serial = serial_id;
   {
     enum GNUNET_DB_QueryStatus qs;
-    struct TALER_EXCHANGEDB_Deposit dep = {
-      .coin.coin_pub = dc->coin_pub,
-      .h_contract_terms = dc->h_contract_terms,
-      .merchant_pub = dc->merchant,
-      .h_wire = dc->h_wire,
-      .refund_deadline = dc->refund_deadline
-    };
-    struct GNUNET_TIME_Absolute exchange_timestamp;
+    struct GNUNET_TIME_Timestamp exchange_timestamp;
     struct TALER_Amount deposit_fee;
 
-    qs = TALER_ARL_edb->have_deposit (TALER_ARL_edb->cls,
-                                      TALER_ARL_esession,
-                                      &dep,
-                                      GNUNET_NO /* do not check refund deadline */,
-                                      &deposit_fee,
-                                      &exchange_timestamp);
+    qs = TALER_ARL_edb->have_deposit2 (TALER_ARL_edb->cls,
+                                       &dc->h_contract_terms,
+                                       &dc->h_wire,
+                                       &dc->coin_pub,
+                                       &dc->merchant,
+                                       dc->refund_deadline,
+                                       &deposit_fee,
+                                       &exchange_timestamp);
     if (qs > 0)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "Found deposit %s in exchange database\n",
-                  GNUNET_h2s (&dc->h_contract_terms));
+                  GNUNET_h2s (&dc->h_contract_terms.hash));
       if (TALER_ARL_do_abort ())
         return GNUNET_SYSERR;
       return GNUNET_OK; /* found, all good */
@@ -141,17 +140,17 @@ test_dc (void *cls,
     }
   }
   /* deposit confirmation missing! report! */
-  TALER_ARL_report (report_deposit_confirmation_inconsistencies,
-                    json_pack ("{s:o, s:o, s:I, s:o}",
-                               "timestamp",
-                               TALER_ARL_json_from_time_abs (
-                                 dc->exchange_timestamp),
-                               "amount",
-                               TALER_JSON_from_amount (&dc->amount_without_fee),
-                               "rowid",
-                               (json_int_t) serial_id,
-                               "account",
-                               GNUNET_JSON_from_data_auto (&dc->h_wire)));
+  TALER_ARL_report (
+    report_deposit_confirmation_inconsistencies,
+    GNUNET_JSON_PACK (
+      TALER_JSON_pack_time_abs_human ("timestamp",
+                                      dc->exchange_timestamp.abs_time),
+      TALER_JSON_pack_amount ("amount",
+                              &dc->amount_without_fee),
+      GNUNET_JSON_pack_uint64 ("rowid",
+                               serial_id),
+      GNUNET_JSON_pack_data_auto ("account",
+                                  &dc->h_wire)));
   dcc->first_missed_coin_serial = GNUNET_MIN (dcc->first_missed_coin_serial,
                                               serial_id);
   dcc->missed_count++;
@@ -186,7 +185,6 @@ analyze_deposit_confirmations (void *cls)
   ppdc.last_deposit_confirmation_serial_id = 0;
   qsp = TALER_ARL_adb->get_auditor_progress_deposit_confirmation (
     TALER_ARL_adb->cls,
-    TALER_ARL_asession,
     &TALER_ARL_master_pub,
     &ppdc);
   if (0 > qsp)
@@ -208,18 +206,17 @@ analyze_deposit_confirmations (void *cls)
 
   /* setup 'cc' */
   GNUNET_assert (GNUNET_OK ==
-                 TALER_amount_get_zero (TALER_ARL_currency,
+                 TALER_amount_set_zero (TALER_ARL_currency,
                                         &dcc.missed_amount));
   dcc.qs = GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
   dcc.missed_count = 0LLU;
   dcc.first_missed_coin_serial = UINT64_MAX;
-  qsx = TALER_ARL_adb->get_deposit_confirmations (TALER_ARL_adb->cls,
-                                                  TALER_ARL_asession,
-                                                  &TALER_ARL_master_pub,
-                                                  ppdc.
-                                                  last_deposit_confirmation_serial_id,
-                                                  &test_dc,
-                                                  &dcc);
+  qsx = TALER_ARL_adb->get_deposit_confirmations (
+    TALER_ARL_adb->cls,
+    &TALER_ARL_master_pub,
+    ppdc.last_deposit_confirmation_serial_id,
+    &test_dc,
+    &dcc);
   if (0 > qsx)
   {
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qsx);
@@ -243,13 +240,11 @@ analyze_deposit_confirmations (void *cls)
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qsp)
     qs = TALER_ARL_adb->update_auditor_progress_deposit_confirmation (
       TALER_ARL_adb->cls,
-      TALER_ARL_asession,
       &TALER_ARL_master_pub,
       &ppdc);
   else
     qs = TALER_ARL_adb->insert_auditor_progress_deposit_confirmation (
       TALER_ARL_adb->cls,
-      TALER_ARL_asession,
       &TALER_ARL_master_pub,
       &ppdc);
   if (0 >= qs)
@@ -291,7 +286,7 @@ run (void *cls,
   if (GNUNET_OK !=
       TALER_ARL_init (c))
   {
-    global_ret = 1;
+    global_ret = EXIT_FAILURE;
     return;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
@@ -302,32 +297,23 @@ run (void *cls,
       TALER_ARL_setup_sessions_and_run (&analyze_deposit_confirmations,
                                         NULL))
   {
-    global_ret = 1;
+    global_ret = EXIT_FAILURE;
     return;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Deposit audit complete\n");
-  {
-    json_t *report;
-
-    report = json_pack ("{s:o, s:I, s:o, s:o, s:o}",
-                        "deposit_confirmation_inconsistencies",
-                        report_deposit_confirmation_inconsistencies,
-                        "missing_deposit_confirmation_count",
-                        (json_int_t) number_missed_deposit_confirmations,
-                        "missing_deposit_confirmation_total",
-                        TALER_JSON_from_amount (
-                          &total_missed_deposit_confirmations),
-                        "auditor_start_time",
-                        TALER_ARL_json_from_time_abs (
-                          start_time),
-                        "auditor_end_time",
-                        TALER_ARL_json_from_time_abs (
-                          GNUNET_TIME_absolute_get ())
-                        );
-    GNUNET_break (NULL != report);
-    TALER_ARL_done (report);
-  }
+  TALER_ARL_done (
+    GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_array_steal ("deposit_confirmation_inconsistencies",
+                                    report_deposit_confirmation_inconsistencies),
+      GNUNET_JSON_pack_uint64 ("missing_deposit_confirmation_count",
+                               number_missed_deposit_confirmations),
+      TALER_JSON_pack_amount ("missing_deposit_confirmation_total",
+                              &total_missed_deposit_confirmations),
+      TALER_JSON_pack_time_abs_human ("auditor_start_time",
+                                      start_time),
+      TALER_JSON_pack_time_abs_human ("auditor_end_time",
+                                      GNUNET_TIME_absolute_get ())));
 }
 
 
@@ -343,6 +329,10 @@ main (int argc,
       char *const *argv)
 {
   const struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_flag ('i',
+                               "internal",
+                               "perform checks only applicable for exchange-internal audits",
+                               &internal_checks),
     GNUNET_GETOPT_option_base32_auto ('m',
                                       "exchange-key",
                                       "KEY",
@@ -352,24 +342,30 @@ main (int argc,
                                      "timetravel"),
     GNUNET_GETOPT_OPTION_END
   };
+  enum GNUNET_GenericReturnValue ret;
 
   /* force linker to link against libtalerutil; if we do
      not do this, the linker may "optimize" libtalerutil
      away and skip #TALER_OS_init(), which we do need */
   (void) TALER_project_data_default ();
-  GNUNET_assert (GNUNET_OK ==
-                 GNUNET_log_setup ("taler-helper-auditor-deposits",
-                                   "MESSAGE",
-                                   NULL));
   if (GNUNET_OK !=
-      GNUNET_PROGRAM_run (argc,
-                          argv,
-                          "taler-helper-auditor-deposits",
-                          "Audit Taler exchange database for deposit confirmation consistency",
-                          options,
-                          &run,
-                          NULL))
-    return 1;
+      GNUNET_STRINGS_get_utf8_args (argc, argv,
+                                    &argc, &argv))
+    return EXIT_INVALIDARGUMENT;
+  ret = GNUNET_PROGRAM_run (
+    argc,
+    argv,
+    "taler-helper-auditor-deposits",
+    gettext_noop (
+      "Audit Taler exchange database for deposit confirmation consistency"),
+    options,
+    &run,
+    NULL);
+  GNUNET_free_nz ((void *) argv);
+  if (GNUNET_SYSERR == ret)
+    return EXIT_INVALIDARGUMENT;
+  if (GNUNET_NO == ret)
+    return EXIT_SUCCESS;
   return global_ret;
 }
 
