@@ -75,12 +75,12 @@ handle_admin_add_incoming_finished (void *cls,
 {
   struct TALER_BANK_AdminAddIncomingHandle *aai = cls;
   uint64_t row_id = UINT64_MAX;
-  struct GNUNET_TIME_Absolute timestamp;
+  struct GNUNET_TIME_Timestamp timestamp;
   enum TALER_ErrorCode ec;
   const json_t *j = response;
 
   aai->job = NULL;
-  timestamp = GNUNET_TIME_UNIT_FOREVER_ABS;
+  timestamp = GNUNET_TIME_UNIT_FOREVER_TS;
   switch (response_code)
   {
   case 0:
@@ -91,8 +91,8 @@ handle_admin_add_incoming_finished (void *cls,
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_uint64 ("row_id",
                                  &row_id),
-        TALER_JSON_spec_absolute_time ("timestamp",
-                                       &timestamp),
+        GNUNET_JSON_spec_timestamp ("timestamp",
+                                    &timestamp),
         GNUNET_JSON_spec_end ()
       };
 
@@ -129,6 +129,11 @@ handle_admin_add_incoming_finished (void *cls,
        We should pass the JSON reply to the application */
     ec = TALER_JSON_get_error_code (j);
     break;
+  case MHD_HTTP_CONFLICT:
+    /* Nothing to verify, we used the same wire subject
+       twice? */
+    ec = TALER_JSON_get_error_code (j);
+    break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
@@ -153,23 +158,6 @@ handle_admin_add_incoming_finished (void *cls,
 }
 
 
-/**
- * Perform a wire transfer from some account to the exchange to fill a
- * reserve.  Note that this API is usually only used for testing (with
- * fakebank and our Python bank) and thus may not be accessible in a
- * production setting.
- *
- * @param ctx curl context for the event loop
- * @param auth authentication data to send to the bank
- * @param reserve_pub wire transfer subject for the transfer
- * @param amount amount that was is to be deposited
- * @param debit_account account to deposit from (payto URI, but used as 'payfrom')
- * @param res_cb the callback to call when the final result for this request is available
- * @param res_cb_cls closure for the above callback
- * @return NULL
- *         if the inputs are invalid (i.e. invalid amount) or internal errors.
- *         In this case, the callback is not called.
- */
 struct TALER_BANK_AdminAddIncomingHandle *
 TALER_BANK_admin_add_incoming (
   struct GNUNET_CURL_Context *ctx,
@@ -184,13 +172,28 @@ TALER_BANK_admin_add_incoming (
   json_t *admin_obj;
   CURL *eh;
 
-  admin_obj = json_pack ("{s:o, s:o, s:s}",
-                         "reserve_pub",
-                         GNUNET_JSON_from_data_auto (reserve_pub),
-                         "amount",
-                         TALER_JSON_from_amount (amount),
-                         "debit_account",
-                         debit_account);
+  if (NULL == debit_account)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  if (NULL == reserve_pub)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  if (NULL == amount)
+  {
+    GNUNET_break (0);
+    return NULL;
+  }
+  admin_obj = GNUNET_JSON_PACK (
+    GNUNET_JSON_pack_data_auto ("reserve_pub",
+                                reserve_pub),
+    TALER_JSON_pack_amount ("amount",
+                            amount),
+    GNUNET_JSON_pack_string ("debit_account",
+                             debit_account));
   if (NULL == admin_obj)
   {
     GNUNET_break (0);
@@ -206,16 +209,16 @@ TALER_BANK_admin_add_incoming (
   {
     GNUNET_free (aai);
     json_decref (admin_obj);
-    GNUNET_break (0);
     return NULL;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Requesting administrative transaction at `%s' for reserve %s\n",
               aai->request_url,
               TALER_B2S (reserve_pub));
-  aai->post_ctx.headers = curl_slist_append
-                            (aai->post_ctx.headers,
-                            "Content-Type: application/json");
+  aai->post_ctx.headers
+    = curl_slist_append (
+        aai->post_ctx.headers,
+        "Content-Type: application/json");
 
   eh = curl_easy_init ();
   if ( (NULL == eh) ||
@@ -249,12 +252,6 @@ TALER_BANK_admin_add_incoming (
 }
 
 
-/**
- * Cancel an add incoming.  This function cannot be used on a request
- * handle if a response is already served for it.
- *
- * @param aai the admin add incoming request handle
- */
 void
 TALER_BANK_admin_add_incoming_cancel (
   struct TALER_BANK_AdminAddIncomingHandle *aai)
