@@ -167,6 +167,11 @@ static enum BenchmarkMode mode;
  */
 static int linger;
 
+/**
+ * Do not initialize or reset the database.
+ */
+static int incremental;
+
 /**
  * Configuration.
  */
@@ -550,6 +555,7 @@ parallel_benchmark (void)
       return GNUNET_SYSERR;
     }
 
+    if (0 == incremental)
     {
       struct GNUNET_OS_Process *dbinit;
 
@@ -559,7 +565,20 @@ parallel_benchmark (void)
                                         "taler-exchange-dbinit",
                                         "-c", cfg_filename,
                                         "-r",
+                                        (NULL != loglev) ? "-L" : NULL,
+                                        loglev,
                                         NULL);
+      if (NULL == dbinit)
+      {
+        if (NULL != bankd)
+        {
+          GNUNET_OS_process_kill (bankd,
+                                  SIGTERM);
+          GNUNET_OS_process_destroy (bankd);
+          bankd = NULL;
+        }
+        return GNUNET_SYSERR;
+      }
       GNUNET_break (GNUNET_OK ==
                     GNUNET_OS_process_wait (dbinit));
       GNUNET_OS_process_destroy (dbinit);
@@ -572,7 +591,8 @@ parallel_benchmark (void)
                                               "taler-exchange-wirewatch",
                                               "taler-exchange-wirewatch",
                                               "-c", cfg_filename,
-                                              "-L", loglev,
+                                              (NULL != loglev) ? "-L" : NULL,
+                                              loglev,
                                               NULL);
       if (NULL == wirewatch[w])
       {
@@ -648,8 +668,9 @@ parallel_benchmark (void)
                                               "taler-exchange-wirewatch",
                                               "taler-exchange-wirewatch",
                                               "-c", cfg_filename,
-                                              "-L", loglev,
                                               "-t",
+                                              (NULL != loglev) ? "-L" : NULL,
+                                              loglev,
                                               NULL);
       /* wait for it to finish! */
       GNUNET_break (GNUNET_OK ==
@@ -673,8 +694,9 @@ parallel_benchmark (void)
                                               "taler-exchange-wirewatch",
                                               "taler-exchange-wirewatch",
                                               "-c", cfg_filename,
-                                              "-L", loglev,
                                               "-t",
+                                              (NULL != loglev) ? "-L" : NULL,
+                                              loglev,
                                               NULL);
       /* wait for it to finish! */
       GNUNET_break (GNUNET_OK ==
@@ -753,6 +775,10 @@ main (int argc,
                                "linger",
                                "linger around until key press",
                                &linger),
+    GNUNET_GETOPT_option_flag ('i',
+                               "incremental",
+                               "skip initializing and resetting the database",
+                               &incremental),
     GNUNET_GETOPT_option_string ('l',
                                  "logfile",
                                  "LF",
