@@ -29,6 +29,7 @@
 #include <sys/resource.h>
 #include "taler_util.h"
 #include "taler_signatures.h"
+#include "taler_exchangedb_lib.h"
 #include "taler_exchange_service.h"
 #include "taler_json_lib.h"
 #include "taler_bank_service.h"
@@ -82,7 +83,7 @@ enum BenchmarkMode
 /**
  * Hold information regarding which bank has the exchange account.
  */
-static struct TALER_BANK_AuthenticationData exchange_bank_account;
+static const struct TALER_EXCHANGEDB_AccountInfo *exchange_bank_account;
 
 /**
  * Configuration of our exchange.
@@ -118,7 +119,7 @@ static char *cfg_filename;
 /**
  * Exit code.
  */
-static int result;
+static enum GNUNET_GenericReturnValue result;
 
 /**
  * Use the fakebank instead of the Python bank.
@@ -252,34 +253,11 @@ static struct TALER_TESTING_Command
 cmd_transfer_to_exchange (const char *label,
                           const char *amount)
 {
-  return TALER_TESTING_cmd_admin_add_incoming_retry
-           (TALER_TESTING_cmd_admin_add_incoming (label,
-                                                  amount,
-                                                  &exchange_bank_account,
-                                                  user_payto_uri));
-}
-
-
-/**
- * Decide which exchange account is going to be
- * used to address a wire transfer to.  Used at
- * withdrawal time.
- *
- * @param cls closure
- * @param section section name.
- */
-static void
-pick_exchange_account_cb (void *cls,
-                          const char *section)
-{
-  if (0 == strncasecmp ("exchange-account-",
-                        section,
-                        strlen ("exchange-account-")))
-  {
-    const char **s = cls;
-
-    *s = section;
-  }
+  return TALER_TESTING_cmd_admin_add_incoming_retry (
+    TALER_TESTING_cmd_admin_add_incoming (label,
+                                          amount,
+                                          exchange_bank_account->auth,
+                                          user_payto_uri));
 }
 
 
@@ -297,7 +275,7 @@ eval_probability (float probability)
 
   random = GNUNET_CRYPTO_random_u64 (GNUNET_CRYPTO_QUALITY_WEAK,
                                      UINT64_MAX);
-  random_01 = (double) random / UINT64_MAX;
+  random_01 = (double) random / (double) UINT64_MAX;
   return (random_01 <= probability) ? GNUNET_OK : GNUNET_NO;
 }
 
@@ -328,7 +306,7 @@ run (void *cls,
   GNUNET_asprintf (&amount_5, "%s:5", currency);
   GNUNET_asprintf (&amount_4, "%s:4", currency);
   GNUNET_asprintf (&amount_1, "%s:1", currency);
-  GNUNET_assert (GNUNET_OK == TALER_amount_get_zero (currency,
+  GNUNET_assert (GNUNET_OK == TALER_amount_set_zero (currency,
                                                      &total_reserve_amount));
   total_reserve_amount.value = 5 * howmany_coins;
   GNUNET_asprintf (&withdraw_fee_str,
@@ -388,6 +366,7 @@ run (void *cls,
           (TALER_TESTING_cmd_withdraw_amount (wl,
                                               create_reserve_label,
                                               amount_5,
+                                              0, /* age restriction off */
                                               MHD_HTTP_OK));
       unit[1] =
         TALER_TESTING_cmd_deposit_with_retry
@@ -459,7 +438,7 @@ run (void *cls,
   GNUNET_free (amount_4);
   GNUNET_free (amount_5);
   GNUNET_free (withdraw_fee_str);
-  result = 1;
+  result = GNUNET_OK;
 }
 
 
@@ -520,8 +499,9 @@ launch_fakebank (void *cls)
 
   (void) cls;
   fakebank
-    = TALER_TESTING_run_fakebank (exchange_bank_account.wire_gateway_url,
-                                  currency);
+    = TALER_TESTING_run_fakebank (
+        exchange_bank_account->auth->wire_gateway_url,
+        currency);
   if (NULL == fakebank)
   {
     GNUNET_break (0);
@@ -546,7 +526,7 @@ parallel_benchmark (TALER_TESTING_Main main_cb,
                     void *main_cb_cls,
                     const char *config_file)
 {
-  int result = GNUNET_OK;
+  enum GNUNET_GenericReturnValue result = GNUNET_OK;
   pid_t cpids[howmany_clients];
   pid_t fakebank = -1;
   int wstatus;
@@ -989,6 +969,8 @@ main (int argc,
                                    argv)))
   {
     GNUNET_free (cfg_filename);
+    if (GNUNET_NO == result)
+      return 0;
     return BAD_CLI_ARG;
   }
   GNUNET_log_setup ("taler-exchange-benchmark",
@@ -1009,8 +991,13 @@ main (int argc,
     return BAD_CONFIG_FILE;
   }
   if (NULL == cfg_filename)
-    cfg_filename = GNUNET_strdup (
-      GNUNET_OS_project_data_get ()->user_config_file);
+    cfg_filename = GNUNET_CONFIGURATION_default_filename ();
+  if (NULL == cfg_filename)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Can't find default configuration file.\n");
+    return EXIT_NOTCONFIGURED;
+  }
   cfg = GNUNET_CONFIGURATION_create ();
   if (GNUNET_OK !=
       GNUNET_CONFIGURATION_load (cfg,
@@ -1052,30 +1039,25 @@ main (int argc,
     return BAD_CONFIG_FILE;
   }
 
+  if (GNUNET_OK !=
+      TALER_EXCHANGEDB_load_accounts (cfg,
+                                      TALER_EXCHANGEDB_ALO_AUTHDATA
+                                      | TALER_EXCHANGEDB_ALO_CREDIT))
   {
-    const char *bank_details_section;
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Configuration fails to provide exchange bank details\n");
+    GNUNET_free (cfg_filename);
+    return BAD_CONFIG_FILE;
+  }
 
-    GNUNET_CONFIGURATION_iterate_sections (cfg,
-                                           &pick_exchange_account_cb,
-                                           &bank_details_section);
-    if (NULL == bank_details_section)
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Missing specification of bank account in configuration\n");
-      GNUNET_free (cfg_filename);
-      return BAD_CONFIG_FILE;
-    }
-    if (GNUNET_OK !=
-        TALER_BANK_auth_parse_cfg (cfg,
-                                   bank_details_section,
-                                   &exchange_bank_account))
-    {
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                  "Configuration fails to provide exchange bank details in section `%s'\n",
-                  bank_details_section);
-      GNUNET_free (cfg_filename);
-      return BAD_CONFIG_FILE;
-    }
+  exchange_bank_account
+    = TALER_EXCHANGEDB_find_account_by_method ("x-taler-bank");
+  if (NULL == exchange_bank_account)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "No bank account for `x-taler-bank` given in configuration\n");
+    GNUNET_free (cfg_filename);
+    return BAD_CONFIG_FILE;
   }
   if ( (MODE_EXCHANGE == mode) || (MODE_BOTH == mode) )
   {
@@ -1154,6 +1136,7 @@ main (int argc,
   result = parallel_benchmark (&run,
                                NULL,
                                cfg_filename);
+  TALER_EXCHANGEDB_unload_accounts ();
   GNUNET_CONFIGURATION_destroy (cfg);
   GNUNET_free (cfg_filename);
 
