@@ -240,6 +240,7 @@ batch_withdraw_transaction (void *cls,
     TEH_plugin->rollback (TEH_plugin->cls);
     *mhd_ret = TEH_RESPONSE_reply_reserve_insufficient_balance (
       connection,
+      TALER_EC_EXCHANGE_WITHDRAW_INSUFFICIENT_FUNDS,
       &wc->batch_total,
       wc->reserve_pub);
     return GNUNET_DB_STATUS_HARD_ERROR;
@@ -412,20 +413,24 @@ static MHD_RESULT
 prepare_transaction (const struct TEH_RequestContext *rc,
                      struct BatchWithdrawContext *wc)
 {
-  /* Note: We could check the reserve balance here,
-     just to be reasonably sure that the reserve has
-     a sufficient balance before doing the "expensive"
-     signatures... */
-  /* Sign before transaction! */
+  struct TEH_CoinSignData csds[wc->planchets_length];
+  struct TALER_BlindedDenominationSignature bss[wc->planchets_length];
+
   for (unsigned int i = 0; i<wc->planchets_length; i++)
   {
     struct PlanchetContext *pc = &wc->planchets[i];
+
+    csds[i].h_denom_pub = &pc->collectable.denom_pub_hash;
+    csds[i].bp = &pc->blinded_planchet;
+  }
+  {
     enum TALER_ErrorCode ec;
 
-    ec = TEH_keys_denomination_sign_withdraw (
-      &pc->collectable.denom_pub_hash,
-      &pc->blinded_planchet,
-      &pc->collectable.sig);
+    ec = TEH_keys_denomination_batch_sign (
+      csds,
+      wc->planchets_length,
+      false,
+      bss);
     if (TALER_EC_NONE != ec)
     {
       GNUNET_break (0);
@@ -434,6 +439,12 @@ prepare_transaction (const struct TEH_RequestContext *rc,
                                       NULL);
     }
   }
+  for (unsigned int i = 0; i<wc->planchets_length; i++)
+  {
+    struct PlanchetContext *pc = &wc->planchets[i];
+
+    pc->collectable.sig = bss[i];
+  }
 
   /* run transaction */
   {
