@@ -166,6 +166,7 @@ deposit_transaction (void *cls,
     struct TEH_PurseDepositedCoin *coin = &pcc->coins[i];
     bool balance_ok = false;
     bool conflict = true;
+    bool too_late = true;
 
     qs = TEH_make_coin_known (&coin->cpi,
                               connection,
@@ -180,18 +181,20 @@ deposit_transaction (void *cls,
                                        &coin->coin_sig,
                                        &coin->amount_minus_fee,
                                        &balance_ok,
+                                       &too_late,
                                        &conflict);
     if (qs <= 0)
     {
       if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
         return qs;
+      GNUNET_break (0 != qs);
       TALER_LOG_WARNING (
         "Failed to store purse deposit information in database\n");
       *mhd_ret = TALER_MHD_reply_with_error (connection,
                                              MHD_HTTP_INTERNAL_SERVER_ERROR,
                                              TALER_EC_GENERIC_DB_STORE_FAILED,
                                              "do purse deposit");
-      return qs;
+      return GNUNET_DB_STATUS_HARD_ERROR;
     }
     if (! balance_ok)
     {
@@ -203,6 +206,16 @@ deposit_transaction (void *cls,
             &coin->cpi.coin_pub);
       return GNUNET_DB_STATUS_HARD_ERROR;
     }
+    if (too_late)
+    {
+      TEH_plugin->rollback (TEH_plugin->cls);
+      *mhd_ret
+        = TALER_MHD_reply_with_ec (
+            connection,
+            TALER_EC_EXCHANGE_PURSE_DEPOSIT_DECIDED_ALREADY,
+            NULL);
+      return GNUNET_DB_STATUS_HARD_ERROR;
+    }
     if (conflict)
     {
       struct TALER_Amount amount;
@@ -361,6 +374,7 @@ TEH_handler_purses_deposit (
     enum GNUNET_DB_QueryStatus qs;
     struct GNUNET_TIME_Timestamp create_timestamp;
     struct GNUNET_TIME_Timestamp merge_timestamp;
+    bool was_deleted;
 
     qs = TEH_plugin->select_purse (
       TEH_plugin->cls,
@@ -370,7 +384,8 @@ TEH_handler_purses_deposit (
       &pcc.amount,
       &pcc.deposit_total,
       &pcc.h_contract_terms,
-      &merge_timestamp);
+      &merge_timestamp,
+      &was_deleted);
     switch (qs)
     {
     case GNUNET_DB_STATUS_HARD_ERROR:
@@ -393,12 +408,16 @@ TEH_handler_purses_deposit (
     case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
       break; /* handled below */
     }
-    if (GNUNET_TIME_absolute_is_past (pcc.purse_expiration.abs_time))
+    if (GNUNET_TIME_absolute_is_past (pcc.purse_expiration.abs_time) ||
+        was_deleted)
     {
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_GONE,
-                                         TALER_EC_EXCHANGE_GENERIC_PURSE_EXPIRED,
-                                         NULL);
+      return TALER_MHD_reply_with_error (
+        connection,
+        MHD_HTTP_GONE,
+        was_deleted
+        ? TALER_EC_EXCHANGE_GENERIC_PURSE_DELETED
+        : TALER_EC_EXCHANGE_GENERIC_PURSE_EXPIRED,
+        GNUNET_TIME_timestamp2s (pcc.purse_expiration));
     }
   }
 
