@@ -704,6 +704,7 @@ TEH_RESPONSE_compile_reserve_history (
   json_t *json_history;
 
   json_history = json_array ();
+  GNUNET_assert (NULL != json_history);
   for (const struct TALER_EXCHANGEDB_ReserveHistory *pos = rh;
        NULL != pos;
        pos = pos->next)
@@ -886,10 +887,10 @@ TEH_RESPONSE_compile_reserve_history (
                                             merge->merge_timestamp),
                 GNUNET_JSON_pack_timestamp ("purse_expiration",
                                             merge->purse_expiration),
-                TALER_JSON_pack_amount ("amount",
-                                        &merge->amount_with_fee),
                 TALER_JSON_pack_amount ("purse_fee",
                                         &merge->purse_fee),
+                TALER_JSON_pack_amount ("amount",
+                                        &merge->amount_with_fee),
                 GNUNET_JSON_pack_bool ("merged",
                                        merge->merged))))
         {
@@ -994,14 +995,16 @@ TEH_RESPONSE_compile_reserve_history (
  * requested withdraw operation.
  *
  * @param connection connection to the client
+ * @param ec error code to return
  * @param ebalance expected balance based on our database
  * @param withdraw_amount amount that the client requested to withdraw
  * @param rh reserve history to return
  * @return MHD result code
  */
 static MHD_RESULT
-reply_withdraw_insufficient_funds (
+reply_reserve_insufficient_funds (
   struct MHD_Connection *connection,
+  enum TALER_ErrorCode ec,
   const struct TALER_Amount *ebalance,
   const struct TALER_Amount *withdraw_amount,
   const struct TALER_EXCHANGEDB_ReserveHistory *rh)
@@ -1010,14 +1013,18 @@ reply_withdraw_insufficient_funds (
 
   json_history = TEH_RESPONSE_compile_reserve_history (rh);
   if (NULL == json_history)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Failed to compile reserve history\n");
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                       TALER_EC_EXCHANGE_WITHDRAW_HISTORY_ERROR_INSUFFICIENT_FUNDS,
+                                       TALER_EC_EXCHANGE_RESERVE_HISTORY_ERROR_INSUFFICIENT_FUNDS,
                                        NULL);
+  }
   return TALER_MHD_REPLY_JSON_PACK (
     connection,
     MHD_HTTP_CONFLICT,
-    TALER_JSON_pack_ec (TALER_EC_EXCHANGE_WITHDRAW_INSUFFICIENT_FUNDS),
+    TALER_JSON_pack_ec (ec),
     TALER_JSON_pack_amount ("balance",
                             ebalance),
     TALER_JSON_pack_amount ("requested_amount",
@@ -1030,6 +1037,7 @@ reply_withdraw_insufficient_funds (
 MHD_RESULT
 TEH_RESPONSE_reply_reserve_insufficient_balance (
   struct MHD_Connection *connection,
+  enum TALER_ErrorCode ec,
   const struct TALER_Amount *balance_required,
   const struct TALER_ReservePublicKeyP *reserve_pub)
 {
@@ -1058,13 +1066,15 @@ TEH_RESPONSE_reply_reserve_insufficient_balance (
   if ( (qs < 0) ||
        (NULL == rh) )
   {
+    GNUNET_break (0);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
                                        TALER_EC_GENERIC_DB_FETCH_FAILED,
                                        "reserve history");
   }
-  mhd_ret = reply_withdraw_insufficient_funds (
+  mhd_ret = reply_reserve_insufficient_funds (
     connection,
+    ec,
     &balance,
     balance_required,
     rh);
@@ -1124,6 +1134,7 @@ TEH_RESPONSE_reply_kyc_required (struct MHD_Connection *connection,
   return TALER_MHD_REPLY_JSON_PACK (
     connection,
     MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS,
+    TALER_JSON_pack_ec (TALER_EC_EXCHANGE_GENERIC_KYC_REQUIRED),
     GNUNET_JSON_pack_data_auto ("h_payto",
                                 h_payto),
     GNUNET_JSON_pack_uint64 ("requirement_row",
