@@ -215,8 +215,11 @@ withdraw_transaction (void *cls,
   if (! balance_ok)
   {
     TEH_plugin->rollback (TEH_plugin->cls);
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Balance insufficient for /withdraw\n");
     *mhd_ret = TEH_RESPONSE_reply_reserve_insufficient_balance (
       connection,
+      TALER_EC_EXCHANGE_WITHDRAW_INSUFFICIENT_FUNDS,
       &wc->collectable.amount_with_fee,
       &wc->collectable.reserve_pub);
     return GNUNET_DB_STATUS_HARD_ERROR;
@@ -447,11 +450,18 @@ TEH_handler_withdraw (struct TEH_RequestContext *rc,
                                        NULL);
   }
 
-  /* Sign before transaction! */
-  ec = TEH_keys_denomination_sign_withdraw (
-    &wc.collectable.denom_pub_hash,
-    &wc.blinded_planchet,
-    &wc.collectable.sig);
+  {
+    struct TEH_CoinSignData csd = {
+      .h_denom_pub = &wc.collectable.denom_pub_hash,
+      .bp = &wc.blinded_planchet
+    };
+
+    /* Sign before transaction! */
+    ec = TEH_keys_denomination_sign (
+      &csd,
+      false,
+      &wc.collectable.sig);
+  }
   if (TALER_EC_NONE != ec)
   {
     GNUNET_break (0);
