@@ -1245,13 +1245,9 @@ run (void *cls)
   }
   (void) plugin->drop_tables (plugin->cls);
   if (GNUNET_OK !=
-      plugin->create_tables (plugin->cls))
-  {
-    result = 77;
-    goto cleanup;
-  }
-  if (GNUNET_OK !=
-      plugin->setup_partitions (plugin->cls, num_partitions))
+      plugin->create_tables (plugin->cls,
+                             true,
+                             num_partitions))
   {
     result = 77;
     goto cleanup;
@@ -1355,6 +1351,7 @@ run (void *cls)
     RND_BLK (&age_hash);
     for (size_t i = 0; i < sizeof(p_ah) / sizeof(p_ah[0]); i++)
     {
+
       RND_BLK (&coin_pub);
       GNUNET_assert (GNUNET_OK ==
                      TALER_denom_blind (&dkp->pub,
