@@ -303,7 +303,6 @@ struct TALER_KYCLOGIC_Plugin
    *
    * @param cls the @e cls of this struct with the plugin-specific state
    * @param pd provider configuration details
-   * @param url_path rest of the URL after `/kyc-webhook/$H_PAYTO/$LOGIC`
    * @param connection MHD connection object (for HTTP headers)
    * @param account_id which account to trigger process for
    * @param process_row row in the legitimization processes table the legitimization is for
@@ -316,7 +315,6 @@ struct TALER_KYCLOGIC_Plugin
   struct TALER_KYCLOGIC_ProofHandle *
   (*proof)(void *cls,
            const struct TALER_KYCLOGIC_ProviderDetails *pd,
-           const char *const url_path[],
            struct MHD_Connection *connection,
            const struct TALER_PaytoHashP *account_id,
            uint64_t process_row,
