@@ -27,9 +27,14 @@
 #ifndef TALER_SIGNATURES_H
 #define TALER_SIGNATURES_H
 
-#include <gnunet/gnunet_util_lib.h>
-#include "taler_amount_lib.h"
-#include "taler_crypto_lib.h"
+
+
+
+
+/**
+ * Initialize or update the status of an AML key for an AML officer
+ */
+#define TALER_SIGNATURE_MASTER_AML_KEY 1017
 
 
 /**
@@ -380,6 +385,12 @@
 #define TALER_SIGNATURE_WALLET_RESERVE_ATTEST_DETAILS 1219
 
 
+/**
+ * Signature by which a wallet requests a purse to be deleted.
+ */
+#define TALER_SIGNATURE_WALLET_PURSE_DELETE 1220
+
+
 /**
  * Signature on a denomination key announcement.
  */
@@ -410,6 +421,12 @@
 #define TALER_SIGNATURE_EXCHANGE_TEST_EDDSA 1303
 
 
+/**
+ * Signature by which an AML officer signs an AML decision.
+ */
+#define TALER_SIGNATURE_AML_DECISION 1350
+
+
 /**
  * EdDSA signature for a policy upload.
  */
