@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2018-2022 Taler Systems SA
+  (C) 2018-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -868,6 +868,24 @@ struct GNUNET_OS_Process *
 TALER_TESTING_run_bank (const char *config_filename,
                         const char *bank_url);
 
+
+/**
+ * Prepare libeufin sandbox execution.  Check if the port is available and
+ * reset database.
+ *
+ * @param config_filename configuration file name.
+ * @param reset_db should we reset the bank's database
+ * @param config_section which configuration section should be used
+ * @param[out] bc set to the bank's configuration data
+ * @return #GNUNET_OK on success
+ */
+enum GNUNET_GenericReturnValue
+TALER_TESTING_prepare_libeufin (const char *config_filename,
+                                bool reset_db,
+                                const char *config_section,
+                                struct TALER_TESTING_BankConfiguration *bc);
+
+
 /**
  * Start the (nexus) bank process.  Assume the port
  * is available and the database is clean.  Use the "prepare
@@ -909,7 +927,7 @@ TALER_TESTING_run_fakebank (const char *bank_url,
  */
 enum GNUNET_GenericReturnValue
 TALER_TESTING_prepare_bank (const char *config_filename,
-                            int reset_db,
+                            bool reset_db,
                             const char *config_section,
                             struct TALER_TESTING_BankConfiguration *bc);
 
@@ -2517,7 +2535,6 @@ TALER_TESTING_cmd_check_kyc_get (const char *label,
  * @param logic_section name of the KYC provider section
  *         in the exchange configuration for this proof
  * @param code OAuth 2.0 code to use
- * @param state OAuth 2.0 state to use
  * @param expected_response_code expected HTTP status
  * @return the command
  */
@@ -2527,7 +2544,6 @@ TALER_TESTING_cmd_proof_kyc_oauth2 (
   const char *payment_target_reference,
   const char *logic_section,
   const char *code,
-  const char *state,
   unsigned int expected_response_code);
 
 
@@ -2567,6 +2583,21 @@ TALER_TESTING_cmd_purse_create_with_deposit (
   ...);
 
 
+/**
+ * Deletes a purse.
+ *
+ * @param label command label
+ * @param expected_http_status what HTTP status do we expect to get returned from the exchange
+ * @param purse_cmd command that created the purse
+ * @return the command
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_purse_delete (
+  const char *label,
+  unsigned int expected_http_status,
+  const char *purse_cmd);
+
+
 /**
  * Retrieve contract (also checks that the contract matches
  * the upload command).
@@ -2679,6 +2710,47 @@ TALER_TESTING_cmd_purse_deposit_coins (
   ...);
 
 
+/**
+ * Setup AML officer.
+ *
+ * @param label command label
+ * @param ref_cmd command that previously created the
+ *       officer, NULL to create one this time
+ * @param name full legal name of the officer to use
+ * @param is_active true to set the officer to active
+ * @param read_only true to restrict the officer to read-only
+ * @return the command
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_set_officer (
+  const char *label,
+  const char *ref_cmd,
+  const char *name,
+  bool is_active,
+  bool read_only);
+
+
+/**
+ * Make AML decision.
+ *
+ * @param label command label
+ * @param ref_officer command that previously created an
+ *       officer
+ * @param ref_operation command that previously created an
+ *       h_payto which to make an AML decision about
+ * @param new_threshold new threshold to set
+ * @param block set to true to block the account
+ * @return the command
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_take_aml_decision (
+  const char *label,
+  const char *ref_officer,
+  const char *ref_operation,
+  const char *new_threshold,
+  bool block);
+
+
 /* *** Generic trait logic for implementing traits ********* */
 
 
