@@ -474,15 +474,14 @@ initiate_task (void *cls)
   hps = GNUNET_STRINGS_data_to_string_alloc (&ih->h_payto,
                                              sizeof (ih->h_payto));
   GNUNET_asprintf (&redirect_uri,
-                   "%s/kyc-proof/%s/%s/%s",
+                   "%skyc-proof/%s?state=%s",
                    ps->exchange_base_url,
-                   hps,
                    pd->section,
-                   legi_s);
+                   hps);
   redirect_uri_encoded = TALER_urlencode (redirect_uri);
   GNUNET_free (redirect_uri);
   GNUNET_asprintf (&url,
-                   "%s?client_id=%s&redirect_uri=%s",
+                   "%s?response_type=code&client_id=%s&redirect_uri=%s",
                    pd->login_url,
                    pd->client_id,
                    redirect_uri_encoded);
@@ -611,7 +610,7 @@ handle_proof_error (struct TALER_KYCLOGIC_ProofHandle *ph,
       ph->response
         = TALER_MHD_make_error (
             TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_INVALID_RESPONSE,
-            "Unexpected response from KYC gateway");
+            "Unexpected response from KYC gateway: proof error");
       ph->http_status
         = MHD_HTTP_BAD_GATEWAY;
       return;
@@ -679,7 +678,7 @@ parse_proof_success_reply (struct TALER_KYCLOGIC_ProofHandle *ph,
     ph->response
       = TALER_MHD_make_error (
           TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_INVALID_RESPONSE,
-          "Unexpected response from KYC gateway");
+          "Unexpected response from KYC gateway: proof success must contain data and status");
     ph->http_status
       = MHD_HTTP_BAD_GATEWAY;
     return;
@@ -714,7 +713,7 @@ parse_proof_success_reply (struct TALER_KYCLOGIC_ProofHandle *ph,
       ph->response
         = TALER_MHD_make_error (
             TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_INVALID_RESPONSE,
-            "Unexpected response from KYC gateway");
+            "Unexpected response from KYC gateway: data must contain id");
       ph->http_status
         = MHD_HTTP_BAD_GATEWAY;
       return;
@@ -797,15 +796,23 @@ handle_curl_login_finished (void *cls,
       const char *token_type;
       uint64_t expires_in_s;
       const char *refresh_token;
+      bool no_expires;
+      bool no_refresh;
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_string ("access_token",
                                  &access_token),
         GNUNET_JSON_spec_string ("token_type",
                                  &token_type),
-        GNUNET_JSON_spec_uint64 ("expires_in",
-                                 &expires_in_s),
-        GNUNET_JSON_spec_string ("refresh_token",
-                                 &refresh_token),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_uint64 ("expires_in",
+                                   &expires_in_s),
+          &no_expires
+          ),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_string ("refresh_token",
+                                   &refresh_token),
+          &no_refresh
+          ),
         GNUNET_JSON_spec_end ()
       };
       CURL *eh;
@@ -825,7 +832,7 @@ handle_curl_login_finished (void *cls,
           ph->response
             = TALER_MHD_make_error (
                 TALER_EC_EXCHANGE_KYC_PROOF_BACKEND_INVALID_RESPONSE,
-                "Unexpected response from KYC gateway");
+                "Unexpected response from KYC gateway: login finished");
           ph->http_status
             = MHD_HTTP_BAD_GATEWAY;
           break;
@@ -918,7 +925,6 @@ handle_curl_login_finished (void *cls,
  *
  * @param cls the @e cls of this struct with the plugin-specific state
  * @param pd provider configuration details
- * @param url_path rest of the URL after `/kyc-webhook/`
  * @param connection MHD connection object (for HTTP headers)
  * @param account_id which account to trigger process for
  * @param process_row row in the legitimization processes table the legitimization is for
@@ -931,7 +937,6 @@ handle_curl_login_finished (void *cls,
 static struct TALER_KYCLOGIC_ProofHandle *
 oauth2_proof (void *cls,
               const struct TALER_KYCLOGIC_ProviderDetails *pd,
-              const char *const url_path[],
               struct MHD_Connection *connection,
               const struct TALER_PaytoHashP *account_id,
               uint64_t process_row,
@@ -944,7 +949,6 @@ oauth2_proof (void *cls,
   struct TALER_KYCLOGIC_ProofHandle *ph;
   const char *code;
 
-  (void) url_path;
   GNUNET_break (NULL == provider_user_id);
   ph = GNUNET_new (struct TALER_KYCLOGIC_ProofHandle);
   GNUNET_snprintf (ph->provider_legitimization_id,
@@ -959,6 +963,7 @@ oauth2_proof (void *cls,
     GNUNET_free (ph);
     return NULL;
   }
+
   ph->pd = pd;
   ph->connection = connection;
   ph->h_payto = *account_id;
@@ -1007,24 +1012,23 @@ oauth2_proof (void *cls,
     char *redirect_uri;
     char *client_secret;
     char *authorization_code;
+    char *redirect_uri_encoded;
+    char *hps;
 
+    hps = GNUNET_STRINGS_data_to_string_alloc (&ph->h_payto,
+                                               sizeof (ph->h_payto));
+    GNUNET_asprintf (&redirect_uri,
+                     "%skyc-proof/%s?state=%s",
+                     ps->exchange_base_url,
+                     pd->section,
+                     hps);
+    redirect_uri_encoded = TALER_urlencode (redirect_uri);
+    GNUNET_free (redirect_uri);
+    GNUNET_assert (NULL != redirect_uri_encoded);
     client_id = curl_easy_escape (ph->eh,
                                   pd->client_id,
                                   0);
     GNUNET_assert (NULL != client_id);
-    {
-      char *request_uri;
-
-      GNUNET_asprintf (&request_uri,
-                       "%s?client_id=%s",
-                       pd->login_url,
-                       pd->client_id);
-      redirect_uri = curl_easy_escape (ph->eh,
-                                       request_uri,
-                                       0);
-      GNUNET_free (request_uri);
-    }
-    GNUNET_assert (NULL != redirect_uri);
     client_secret = curl_easy_escape (ph->eh,
                                       pd->client_secret,
                                       0);
@@ -1036,12 +1040,13 @@ oauth2_proof (void *cls,
     GNUNET_asprintf (&ph->post_body,
                      "client_id=%s&redirect_uri=%s&client_secret=%s&code=%s&grant_type=authorization_code",
                      client_id,
-                     redirect_uri,
+                     redirect_uri_encoded,
                      client_secret,
                      authorization_code);
     curl_free (authorization_code);
     curl_free (client_secret);
-    curl_free (redirect_uri);
+    GNUNET_free (redirect_uri_encoded);
+    GNUNET_free (hps);
     curl_free (client_id);
   }
   GNUNET_assert (CURLE_OK ==
