@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2014-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -120,6 +120,13 @@ handle_deposit_confirmation_finished (void *cls,
     /* Nothing really to verify, this should never
        happen, we should pass the JSON reply to the application */
     break;
+  case MHD_HTTP_GONE:
+    hr.ec = TALER_JSON_get_error_code (json);
+    hr.hint = TALER_JSON_get_error_hint (json);
+    /* Nothing really to verify, auditor says one of the signatures is
+       invalid; as we checked them, this should never happen, we
+       should pass the JSON reply to the application */
+    break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
     hr.ec = TALER_JSON_get_error_code (json);
     hr.hint = TALER_JSON_get_error_hint (json);
@@ -131,10 +138,9 @@ handle_deposit_confirmation_finished (void *cls,
     hr.ec = TALER_JSON_get_error_code (json);
     hr.hint = TALER_JSON_get_error_hint (json);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d\n",
+                "Unexpected response code %u/%d for auditor deposit confirmation\n",
                 (unsigned int) response_code,
                 hr.ec);
-    GNUNET_break_op (0);
     break;
   }
   dh->cb (dh->cb_cls,
@@ -147,8 +153,10 @@ handle_deposit_confirmation_finished (void *cls,
  * Verify signature information about the deposit-confirmation.
  *
  * @param h_wire hash of merchant wire details
+ * @param h_policy hash over the policy extension, if any
  * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the auditor)
  * @param exchange_timestamp timestamp when the deposit was received by the wallet
+ * @param wire_deadline by what time must the amount be wired to the merchant
  * @param refund_deadline date until which the merchant can issue a refund to the customer via the auditor (can be zero if refunds are not allowed); must not be after the @a wire_deadline
  * @param amount_without_fee the amount confirmed to be wired by the exchange to the merchant
  * @param coin_pub coin’s public key
@@ -162,75 +170,62 @@ handle_deposit_confirmation_finished (void *cls,
  * @param master_sig master signature affirming validity of @a exchange_pub
  * @return #GNUNET_OK if signatures are OK, #GNUNET_SYSERR if not
  */
-static int
-verify_signatures (const struct GNUNET_HashCode *h_wire,
-                   const struct GNUNET_HashCode *h_contract_terms,
-                   struct GNUNET_TIME_Absolute exchange_timestamp,
-                   struct GNUNET_TIME_Absolute refund_deadline,
+static enum GNUNET_GenericReturnValue
+verify_signatures (const struct TALER_MerchantWireHashP *h_wire,
+                   const struct TALER_ExtensionPolicyHashP *h_policy,
+                   const struct TALER_PrivateContractHashP *h_contract_terms,
+                   struct GNUNET_TIME_Timestamp exchange_timestamp,
+                   struct GNUNET_TIME_Timestamp wire_deadline,
+                   struct GNUNET_TIME_Timestamp refund_deadline,
                    const struct TALER_Amount *amount_without_fee,
                    const struct TALER_CoinSpendPublicKeyP *coin_pub,
                    const struct TALER_MerchantPublicKeyP *merchant_pub,
                    const struct TALER_ExchangePublicKeyP *exchange_pub,
                    const struct TALER_ExchangeSignatureP *exchange_sig,
                    const struct TALER_MasterPublicKeyP *master_pub,
-                   struct GNUNET_TIME_Absolute ep_start,
-                   struct GNUNET_TIME_Absolute ep_expire,
-                   struct GNUNET_TIME_Absolute ep_end,
+                   struct GNUNET_TIME_Timestamp ep_start,
+                   struct GNUNET_TIME_Timestamp ep_expire,
+                   struct GNUNET_TIME_Timestamp ep_end,
                    const struct TALER_MasterSignatureP *master_sig)
 {
+  if (GNUNET_OK !=
+      TALER_exchange_online_deposit_confirmation_verify (
+        h_contract_terms,
+        h_wire,
+        h_policy,
+        exchange_timestamp,
+        wire_deadline,
+        refund_deadline,
+        amount_without_fee,
+        coin_pub,
+        merchant_pub,
+        exchange_pub,
+        exchange_sig))
   {
-    struct TALER_DepositConfirmationPS dc = {
-      .purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT),
-      .purpose.size = htonl (sizeof (dc)),
-      .h_contract_terms = *h_contract_terms,
-      .h_wire = *h_wire,
-      .exchange_timestamp = GNUNET_TIME_absolute_hton (exchange_timestamp),
-      .refund_deadline = GNUNET_TIME_absolute_hton (refund_deadline),
-      .coin_pub = *coin_pub,
-      .merchant = *merchant_pub
-    };
-
-    TALER_amount_hton (&dc.amount_without_fee,
-                       amount_without_fee);
-    if (GNUNET_OK !=
-        GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT,
-                                    &dc,
-                                    &exchange_sig->eddsa_signature,
-                                    &exchange_pub->eddsa_pub))
+    GNUNET_break_op (0);
+    TALER_LOG_WARNING (
+      "Invalid signature on /deposit-confirmation request!\n");
     {
-      GNUNET_break_op (0);
-      TALER_LOG_WARNING (
-        "Invalid signature on /deposit-confirmation request!\n");
-      {
-        TALER_LOG_DEBUG ("... amount_without_fee was %s\n",
-                         TALER_amount2s (amount_without_fee));
-      }
-      return GNUNET_SYSERR;
+      TALER_LOG_DEBUG ("... amount_without_fee was %s\n",
+                       TALER_amount2s (amount_without_fee));
     }
+    return GNUNET_SYSERR;
   }
+
+  if (GNUNET_OK !=
+      TALER_exchange_offline_signkey_validity_verify (
+        exchange_pub,
+        ep_start,
+        ep_expire,
+        ep_end,
+        master_pub,
+        master_sig))
   {
-    struct TALER_ExchangeSigningKeyValidityPS sv = {
-      .purpose.purpose = htonl (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY),
-      .purpose.size = htonl (sizeof (sv)),
-      .master_public_key = *master_pub,
-      .start = GNUNET_TIME_absolute_hton (ep_start),
-      .expire = GNUNET_TIME_absolute_hton (ep_expire),
-      .end = GNUNET_TIME_absolute_hton (ep_end),
-      .signkey_pub = *exchange_pub
-    };
-
-    if (GNUNET_OK !=
-        GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_MASTER_SIGNING_KEY_VALIDITY,
-                                    &sv,
-                                    &master_sig->eddsa_signature,
-                                    &master_pub->eddsa_pub))
-    {
-      GNUNET_break (0);
-      TALER_LOG_WARNING ("Invalid signature on exchange signing key!\n");
-      return GNUNET_SYSERR;
-    }
+    GNUNET_break (0);
+    TALER_LOG_WARNING ("Invalid signature on exchange signing key!\n");
+    return GNUNET_SYSERR;
   }
-  if (0 == GNUNET_TIME_absolute_get_remaining (ep_end).rel_value_us)
+  if (GNUNET_TIME_absolute_is_past (ep_end.abs_time))
   {
     GNUNET_break (0);
     TALER_LOG_WARNING ("Exchange signing key is no longer valid!\n");
@@ -240,55 +235,24 @@ verify_signatures (const struct GNUNET_HashCode *h_wire,
 }
 
 
-/**
- * Submit a deposit-confirmation permission to the auditor and get the
- * auditor's response.  Note that while we return the response
- * verbatim to the caller for further processing, we do already verify
- * that the response is well-formed.  If the auditor's reply is not
- * well-formed, we return an HTTP status code of zero to @a cb.
- *
- * We also verify that the @a exchange_sig is valid for this deposit-confirmation
- * request, and that the @a master_sig is a valid signature for @a
- * exchange_pub.  Also, the @a auditor must be ready to operate (i.e.  have
- * finished processing the /version reply).  If either check fails, we do
- * NOT initiate the transaction with the auditor and instead return NULL.
- *
- * @param auditor the auditor handle; the auditor must be ready to operate
- * @param h_wire hash of merchant wire details
- * @param h_contract_terms hash of the contact of the merchant with the customer (further details are never disclosed to the auditor)
- * @param exchange_timestamp timestamp when deposit was received by the exchange
- * @param refund_deadline date until which the merchant can issue a refund to the customer via the auditor (can be zero if refunds are not allowed); must not be after the @a wire_deadline
- * @param amount_without_fee the amount confirmed to be wired by the exchange to the merchant
- * @param coin_pub coin’s public key
- * @param merchant_pub the public key of the merchant (used to identify the merchant for refund requests)
- * @param exchange_sig the signature made with purpose #TALER_SIGNATURE_EXCHANGE_CONFIRM_DEPOSIT
- * @param exchange_pub the public key of the exchange that matches @a exchange_sig
- * @param master_pub master public key of the exchange
- * @param ep_start when does @a exchange_pub validity start
- * @param ep_expire when does @a exchange_pub usage end
- * @param ep_end when does @a exchange_pub legal validity end
- * @param master_sig master signature affirming validity of @a exchange_pub
- * @param cb the callback to call when a reply for this request is available
- * @param cb_cls closure for the above callback
- * @return a handle for this request; NULL if the inputs are invalid (i.e.
- *         signatures fail to verify).  In this case, the callback is not called.
- */
 struct TALER_AUDITOR_DepositConfirmationHandle *
 TALER_AUDITOR_deposit_confirmation (
   struct TALER_AUDITOR_Handle *auditor,
-  const struct GNUNET_HashCode *h_wire,
-  const struct GNUNET_HashCode *h_contract_terms,
-  struct GNUNET_TIME_Absolute exchange_timestamp,
-  struct GNUNET_TIME_Absolute refund_deadline,
+  const struct TALER_MerchantWireHashP *h_wire,
+  const struct TALER_ExtensionPolicyHashP *h_policy,
+  const struct TALER_PrivateContractHashP *h_contract_terms,
+  struct GNUNET_TIME_Timestamp exchange_timestamp,
+  struct GNUNET_TIME_Timestamp wire_deadline,
+  struct GNUNET_TIME_Timestamp refund_deadline,
   const struct TALER_Amount *amount_without_fee,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
   const struct TALER_MerchantPublicKeyP *merchant_pub,
   const struct TALER_ExchangePublicKeyP *exchange_pub,
   const struct TALER_ExchangeSignatureP *exchange_sig,
   const struct TALER_MasterPublicKeyP *master_pub,
-  struct GNUNET_TIME_Absolute ep_start,
-  struct GNUNET_TIME_Absolute ep_expire,
-  struct GNUNET_TIME_Absolute ep_end,
+  struct GNUNET_TIME_Timestamp ep_start,
+  struct GNUNET_TIME_Timestamp ep_expire,
+  struct GNUNET_TIME_Timestamp ep_end,
   const struct TALER_MasterSignatureP *master_sig,
   TALER_AUDITOR_DepositConfirmationResultCallback cb,
   void *cb_cls)
@@ -298,17 +262,14 @@ TALER_AUDITOR_deposit_confirmation (
   json_t *deposit_confirmation_obj;
   CURL *eh;
 
-  (void) GNUNET_TIME_round_abs (&exchange_timestamp);
-  (void) GNUNET_TIME_round_abs (&refund_deadline);
-  (void) GNUNET_TIME_round_abs (&ep_start);
-  (void) GNUNET_TIME_round_abs (&ep_expire);
-  (void) GNUNET_TIME_round_abs (&ep_end);
   GNUNET_assert (GNUNET_YES ==
                  TALER_AUDITOR_handle_is_ready_ (auditor));
   if (GNUNET_OK !=
       verify_signatures (h_wire,
+                         h_policy,
                          h_contract_terms,
                          exchange_timestamp,
+                         wire_deadline,
                          refund_deadline,
                          amount_without_fee,
                          coin_pub,
@@ -326,43 +287,50 @@ TALER_AUDITOR_deposit_confirmation (
   }
 
   deposit_confirmation_obj
-    = json_pack ("{s:o, s:o," /* h_wire, h_contract_terms */
-                 " s:o, s:o," /* timestamp, refund_deadline */
-                 " s:o, s:o," /* amount_without_fees, coin_pub */
-                 " s:o, s:o," /* merchant_pub, exchange_sig */
-                 " s:o, s:o," /* master_pub, ep_start */
-                 " s:o, s:o," /* ep_expire, ep_end */
-                 " s:o, s:o}", /* master_sig, exchange_pub */
-                 "h_wire", GNUNET_JSON_from_data_auto (h_wire),
-                 "h_contract_terms", GNUNET_JSON_from_data_auto (
-                   h_contract_terms),
-                 "exchange_timestamp", GNUNET_JSON_from_time_abs (
-                   exchange_timestamp),
-                 "refund_deadline", GNUNET_JSON_from_time_abs (refund_deadline),
-                 "amount_without_fee", TALER_JSON_from_amount (
-                   amount_without_fee),
-                 "coin_pub", GNUNET_JSON_from_data_auto (coin_pub),
-                 "merchant_pub", GNUNET_JSON_from_data_auto (merchant_pub),
-                 "exchange_sig", GNUNET_JSON_from_data_auto (exchange_sig),
-                 "master_pub", GNUNET_JSON_from_data_auto (master_pub),
-                 "ep_start", GNUNET_JSON_from_time_abs (ep_start),
-                 "ep_expire", GNUNET_JSON_from_time_abs (ep_expire),
-                 "ep_end", GNUNET_JSON_from_time_abs (ep_end),
-                 "master_sig", GNUNET_JSON_from_data_auto (master_sig),
-                 "exchange_pub", GNUNET_JSON_from_data_auto (exchange_pub));
-
-  if (NULL == deposit_confirmation_obj)
-  {
-    GNUNET_break (0);
-    return NULL;
-  }
-
+    = GNUNET_JSON_PACK (
+        GNUNET_JSON_pack_data_auto ("h_wire",
+                                    h_wire),
+        GNUNET_JSON_pack_data_auto ("h_policy",
+                                    h_policy),
+        GNUNET_JSON_pack_data_auto ("h_contract_terms",
+                                    h_contract_terms),
+        GNUNET_JSON_pack_timestamp ("exchange_timestamp",
+                                    exchange_timestamp),
+        GNUNET_JSON_pack_timestamp ("refund_deadline",
+                                    refund_deadline),
+        GNUNET_JSON_pack_timestamp ("wire_deadline",
+                                    wire_deadline),
+        TALER_JSON_pack_amount ("amount_without_fee",
+                                amount_without_fee),
+        GNUNET_JSON_pack_data_auto ("coin_pub",
+                                    coin_pub),
+        GNUNET_JSON_pack_data_auto ("merchant_pub",
+                                    merchant_pub),
+        GNUNET_JSON_pack_data_auto ("exchange_sig",
+                                    exchange_sig),
+        GNUNET_JSON_pack_data_auto ("master_pub",
+                                    master_pub),
+        GNUNET_JSON_pack_timestamp ("ep_start",
+                                    ep_start),
+        GNUNET_JSON_pack_timestamp ("ep_expire",
+                                    ep_expire),
+        GNUNET_JSON_pack_timestamp ("ep_end",
+                                    ep_end),
+        GNUNET_JSON_pack_data_auto ("master_sig",
+                                    master_sig),
+        GNUNET_JSON_pack_data_auto ("exchange_pub",
+                                    exchange_pub));
   dh = GNUNET_new (struct TALER_AUDITOR_DepositConfirmationHandle);
   dh->auditor = auditor;
   dh->cb = cb;
   dh->cb_cls = cb_cls;
   dh->url = TALER_AUDITOR_path_to_url_ (auditor,
                                         "/deposit-confirmation");
+  if (NULL == dh->url)
+  {
+    GNUNET_free (dh);
+    return NULL;
+  }
   eh = TALER_AUDITOR_curl_easy_get_ (dh->url);
 
   if ( (NULL == eh) ||
@@ -393,16 +361,14 @@ TALER_AUDITOR_deposit_confirmation (
                                   dh->ctx.headers,
                                   &handle_deposit_confirmation_finished,
                                   dh);
+  /* Disable 100 continue processing */
+  GNUNET_CURL_extend_headers (dh->job,
+                              curl_slist_append (NULL,
+                                                 "Expect:"));
   return dh;
 }
 
 
-/**
- * Cancel a deposit-confirmation permission request.  This function cannot be used
- * on a request handle if a response is already served for it.
- *
- * @param deposit_confirmation the deposit-confirmation permission request handle
- */
 void
 TALER_AUDITOR_deposit_confirmation_cancel (
   struct TALER_AUDITOR_DepositConfirmationHandle *deposit_confirmation)
