@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2020 Taler Systems SA
+  Copyright (C) 2014-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -70,54 +70,24 @@ struct TALER_EXCHANGE_DepositGetHandle
   void *cb_cls;
 
   /**
-   * Information the exchange should sign in response.
-   * (with pre-filled fields from the request).
+   * Hash over the wiring information of the merchant.
    */
-  struct TALER_ConfirmWirePS depconf;
+  struct TALER_MerchantWireHashP h_wire;
+
+  /**
+   * Hash over the contract for which this deposit is made.
+   */
+  struct TALER_PrivateContractHashP h_contract_terms;
+
+  /**
+   * The coin's public key.  This is the value that must have been
+   * signed (blindly) by the Exchange.
+   */
+  struct TALER_CoinSpendPublicKeyP coin_pub;
 
 };
 
 
-/**
- * Verify that the signature on the "200 OK" response
- * from the exchange is valid.
- *
- * @param dwh deposit wtid handle
- * @param json json reply with the signature
- * @param exchange_pub the exchange's public key
- * @param exchange_sig the exchange's signature
- * @return #GNUNET_OK if the signature is valid, #GNUNET_SYSERR if not
- */
-static int
-verify_deposit_wtid_signature_ok (
-  const struct TALER_EXCHANGE_DepositGetHandle *dwh,
-  const json_t *json,
-  const struct TALER_ExchangePublicKeyP *exchange_pub,
-  const struct TALER_ExchangeSignatureP *exchange_sig)
-{
-  const struct TALER_EXCHANGE_Keys *key_state;
-
-  key_state = TALER_EXCHANGE_get_keys (dwh->exchange);
-  if (GNUNET_OK !=
-      TALER_EXCHANGE_test_signing_key (key_state,
-                                       exchange_pub))
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  if (GNUNET_OK !=
-      GNUNET_CRYPTO_eddsa_verify (TALER_SIGNATURE_EXCHANGE_CONFIRM_WIRE,
-                                  &dwh->depconf,
-                                  &exchange_sig->eddsa_signature,
-                                  &exchange_pub->eddsa_pub))
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
-  return GNUNET_OK;
-}
-
-
 /**
  * Function called when we're done processing the
  * HTTP /track/transaction request.
@@ -133,70 +103,89 @@ handle_deposit_wtid_finished (void *cls,
 {
   struct TALER_EXCHANGE_DepositGetHandle *dwh = cls;
   const json_t *j = response;
-  struct TALER_EXCHANGE_HttpResponse hr = {
-    .reply = j,
-    .http_status = (unsigned int) response_code
+  struct TALER_EXCHANGE_GetDepositResponse dr = {
+    .hr.reply = j,
+    .hr.http_status = (unsigned int) response_code
   };
 
   dwh->job = NULL;
   switch (response_code)
   {
   case 0:
-    hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
+    dr.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
     break;
   case MHD_HTTP_OK:
     {
-      struct TALER_EXCHANGE_DepositData dd;
       struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_fixed_auto ("wtid", &dwh->depconf.wtid),
-        TALER_JSON_spec_absolute_time ("execution_time", &dd.execution_time),
-        TALER_JSON_spec_amount ("coin_contribution", &dd.coin_contribution),
-        GNUNET_JSON_spec_fixed_auto ("exchange_sig", &dd.exchange_sig),
-        GNUNET_JSON_spec_fixed_auto ("exchange_pub", &dd.exchange_pub),
+        GNUNET_JSON_spec_fixed_auto ("wtid",
+                                     &dr.details.success.wtid),
+        GNUNET_JSON_spec_timestamp ("execution_time",
+                                    &dr.details.success.execution_time),
+        TALER_JSON_spec_amount_any ("coin_contribution",
+                                    &dr.details.success.coin_contribution),
+        GNUNET_JSON_spec_fixed_auto ("exchange_sig",
+                                     &dr.details.success.exchange_sig),
+        GNUNET_JSON_spec_fixed_auto ("exchange_pub",
+                                     &dr.details.success.exchange_pub),
         GNUNET_JSON_spec_end ()
       };
+      const struct TALER_EXCHANGE_Keys *key_state;
 
+      key_state = TALER_EXCHANGE_get_keys (dwh->exchange);
+      GNUNET_assert (NULL != key_state);
       if (GNUNET_OK !=
           GNUNET_JSON_parse (j,
                              spec,
                              NULL, NULL))
       {
         GNUNET_break_op (0);
-        hr.http_status = 0;
-        hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+        dr.hr.http_status = 0;
+        dr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
         break;
       }
-      dwh->depconf.execution_time = GNUNET_TIME_absolute_hton (
-        dd.execution_time);
-      TALER_amount_hton (&dwh->depconf.coin_contribution,
-                         &dd.coin_contribution);
       if (GNUNET_OK !=
-          verify_deposit_wtid_signature_ok (dwh,
-                                            j,
-                                            &dd.exchange_pub,
-                                            &dd.exchange_sig))
+          TALER_EXCHANGE_test_signing_key (key_state,
+                                           &dr.details.success.exchange_pub))
       {
         GNUNET_break_op (0);
-        hr.http_status = 0;
-        hr.ec = TALER_EC_EXCHANGE_DEPOSITS_GET_INVALID_SIGNATURE_BY_EXCHANGE;
+        dr.hr.http_status = 0;
+        dr.hr.ec = TALER_EC_EXCHANGE_DEPOSITS_GET_INVALID_SIGNATURE_BY_EXCHANGE;
+        break;
       }
-      else
+      if (GNUNET_OK !=
+          TALER_exchange_online_confirm_wire_verify (
+            &dwh->h_wire,
+            &dwh->h_contract_terms,
+            &dr.details.success.wtid,
+            &dwh->coin_pub,
+            dr.details.success.execution_time,
+            &dr.details.success.coin_contribution,
+            &dr.details.success.exchange_pub,
+            &dr.details.success.exchange_sig))
       {
-        dd.wtid = dwh->depconf.wtid;
-        dwh->cb (dwh->cb_cls,
-                 &hr,
-                 &dd);
-        TALER_EXCHANGE_deposits_get_cancel (dwh);
-        return;
+        GNUNET_break_op (0);
+        dr.hr.http_status = 0;
+        dr.hr.ec = TALER_EC_EXCHANGE_DEPOSITS_GET_INVALID_SIGNATURE_BY_EXCHANGE;
+        break;
       }
+      dwh->cb (dwh->cb_cls,
+               &dr);
+      TALER_EXCHANGE_deposits_get_cancel (dwh);
+      return;
     }
-    break;
   case MHD_HTTP_ACCEPTED:
     {
       /* Transaction known, but not executed yet */
-      struct GNUNET_TIME_Absolute execution_time;
+      bool no_legi = false;
       struct GNUNET_JSON_Specification spec[] = {
-        TALER_JSON_spec_absolute_time ("execution_time", &execution_time),
+        GNUNET_JSON_spec_timestamp ("execution_time",
+                                    &dr.details.accepted.execution_time),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_uint64 ("requirement_row",
+                                   &dr.details.accepted.requirement_row),
+          &no_legi),
+        GNUNET_JSON_spec_bool ("kyc_ok",
+                               &dr.details.accepted.kyc_ok),
         GNUNET_JSON_spec_end ()
       };
 
@@ -206,99 +195,78 @@ handle_deposit_wtid_finished (void *cls,
                              NULL, NULL))
       {
         GNUNET_break_op (0);
-        hr.http_status = 0;
-        hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+        dr.hr.http_status = 0;
+        dr.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
         break;
       }
-      else
-      {
-        struct TALER_EXCHANGE_DepositData dd = {
-          .execution_time = execution_time
-        };
-
-        dwh->cb (dwh->cb_cls,
-                 &hr,
-                 &dd);
-        TALER_EXCHANGE_deposits_get_cancel (dwh);
-        return;
-      }
+      if (no_legi)
+        dr.details.accepted.requirement_row = 0;
+      dwh->cb (dwh->cb_cls,
+               &dr);
+      TALER_EXCHANGE_deposits_get_cancel (dwh);
+      return;
     }
-    break;
   case MHD_HTTP_BAD_REQUEST:
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    dr.hr.ec = TALER_JSON_get_error_code (j);
+    dr.hr.hint = TALER_JSON_get_error_hint (j);
     /* This should never happen, either us or the exchange is buggy
        (or API version conflict); just pass JSON reply to the application */
     break;
   case MHD_HTTP_FORBIDDEN:
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    dr.hr.ec = TALER_JSON_get_error_code (j);
+    dr.hr.hint = TALER_JSON_get_error_hint (j);
     /* Nothing really to verify, exchange says one of the signatures is
        invalid; as we checked them, this should never happen, we
        should pass the JSON reply to the application */
     break;
   case MHD_HTTP_NOT_FOUND:
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    dr.hr.ec = TALER_JSON_get_error_code (j);
+    dr.hr.hint = TALER_JSON_get_error_hint (j);
     /* Exchange does not know about transaction;
        we should pass the reply to the application */
     break;
   case MHD_HTTP_INTERNAL_SERVER_ERROR:
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    dr.hr.ec = TALER_JSON_get_error_code (j);
+    dr.hr.hint = TALER_JSON_get_error_hint (j);
     /* Server had an internal issue; we should retry, but this API
        leaves this to the application */
     break;
   default:
     /* unexpected response code */
-    hr.ec = TALER_JSON_get_error_code (j);
-    hr.hint = TALER_JSON_get_error_hint (j);
+    dr.hr.ec = TALER_JSON_get_error_code (j);
+    dr.hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d\n",
+                "Unexpected response code %u/%d for exchange GET deposits\n",
                 (unsigned int) response_code,
-                (int) hr.ec);
+                (int) dr.hr.ec);
     GNUNET_break_op (0);
     break;
   }
   dwh->cb (dwh->cb_cls,
-           &hr,
-           NULL);
+           &dr);
   TALER_EXCHANGE_deposits_get_cancel (dwh);
 }
 
 
-/**
- * Obtain wire transfer details about an existing deposit operation.
- *
- * @param exchange the exchange to query
- * @param merchant_priv the merchant's private key
- * @param h_wire hash of merchant's wire transfer details
- * @param h_contract_terms hash of the proposal data from the contract
- *                        between merchant and customer
- * @param coin_pub public key of the coin
- * @param cb function to call with the result
- * @param cb_cls closure for @a cb
- * @return handle to abort request
- */
 struct TALER_EXCHANGE_DepositGetHandle *
 TALER_EXCHANGE_deposits_get (
   struct TALER_EXCHANGE_Handle *exchange,
   const struct TALER_MerchantPrivateKeyP *merchant_priv,
-  const struct GNUNET_HashCode *h_wire,
-  const struct GNUNET_HashCode *h_contract_terms,
+  const struct TALER_MerchantWireHashP *h_wire,
+  const struct TALER_PrivateContractHashP *h_contract_terms,
   const struct TALER_CoinSpendPublicKeyP *coin_pub,
   TALER_EXCHANGE_DepositGetCallback cb,
   void *cb_cls)
 {
-  struct TALER_DepositTrackPS dtp;
+  struct TALER_MerchantPublicKeyP merchant;
   struct TALER_MerchantSignatureP merchant_sig;
   struct TALER_EXCHANGE_DepositGetHandle *dwh;
   struct GNUNET_CURL_Context *ctx;
   CURL *eh;
   char arg_str[(sizeof (struct TALER_CoinSpendPublicKeyP)
-                + sizeof (struct GNUNET_HashCode)
+                + sizeof (struct TALER_MerchantWireHashP)
                 + sizeof (struct TALER_MerchantPublicKeyP)
-                + sizeof (struct GNUNET_HashCode)
+                + sizeof (struct TALER_PrivateContractHashP)
                 + sizeof (struct TALER_MerchantSignatureP)) * 2 + 48];
 
   if (GNUNET_YES !=
@@ -307,52 +275,43 @@ TALER_EXCHANGE_deposits_get (
     GNUNET_break (0);
     return NULL;
   }
-  dtp.purpose.purpose = htonl (TALER_SIGNATURE_MERCHANT_TRACK_TRANSACTION);
-  dtp.purpose.size = htonl (sizeof (dtp));
-  dtp.h_contract_terms = *h_contract_terms;
-  dtp.h_wire = *h_wire;
   GNUNET_CRYPTO_eddsa_key_get_public (&merchant_priv->eddsa_priv,
-                                      &dtp.merchant.eddsa_pub);
-
-  dtp.coin_pub = *coin_pub;
-  GNUNET_CRYPTO_eddsa_sign (&merchant_priv->eddsa_priv,
-                            &dtp,
-                            &merchant_sig.eddsa_sig);
+                                      &merchant.eddsa_pub);
+  TALER_merchant_deposit_sign (h_contract_terms,
+                               h_wire,
+                               coin_pub,
+                               merchant_priv,
+                               &merchant_sig);
   {
     char cpub_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2];
     char mpub_str[sizeof (struct TALER_MerchantPublicKeyP) * 2];
     char msig_str[sizeof (struct TALER_MerchantSignatureP) * 2];
-    char chash_str[sizeof (struct GNUNET_HashCode) * 2];
-    char whash_str[sizeof (struct GNUNET_HashCode) * 2];
+    char chash_str[sizeof (struct TALER_PrivateContractHashP) * 2];
+    char whash_str[sizeof (struct TALER_MerchantWireHashP) * 2];
     char *end;
 
     end = GNUNET_STRINGS_data_to_string (h_wire,
-                                         sizeof (struct
-                                                 GNUNET_HashCode),
+                                         sizeof (*h_wire),
                                          whash_str,
                                          sizeof (whash_str));
     *end = '\0';
-    end = GNUNET_STRINGS_data_to_string (&dtp.merchant,
-                                         sizeof (struct
-                                                 TALER_MerchantPublicKeyP),
+    end = GNUNET_STRINGS_data_to_string (&merchant,
+                                         sizeof (merchant),
                                          mpub_str,
                                          sizeof (mpub_str));
     *end = '\0';
     end = GNUNET_STRINGS_data_to_string (h_contract_terms,
-                                         sizeof (struct
-                                                 GNUNET_HashCode),
+                                         sizeof (*h_contract_terms),
                                          chash_str,
                                          sizeof (chash_str));
     *end = '\0';
     end = GNUNET_STRINGS_data_to_string (coin_pub,
-                                         sizeof (struct
-                                                 TALER_CoinSpendPublicKeyP),
+                                         sizeof (*coin_pub),
                                          cpub_str,
                                          sizeof (cpub_str));
     *end = '\0';
     end = GNUNET_STRINGS_data_to_string (&merchant_sig,
-                                         sizeof (struct
-                                                 TALER_MerchantSignatureP),
+                                         sizeof (merchant_sig),
                                          msig_str,
                                          sizeof (msig_str));
     *end = '\0';
@@ -373,11 +332,14 @@ TALER_EXCHANGE_deposits_get (
   dwh->cb_cls = cb_cls;
   dwh->url = TEAH_path_to_url (exchange,
                                arg_str);
-  dwh->depconf.purpose.size = htonl (sizeof (struct TALER_ConfirmWirePS));
-  dwh->depconf.purpose.purpose = htonl (TALER_SIGNATURE_EXCHANGE_CONFIRM_WIRE);
-  dwh->depconf.h_wire = *h_wire;
-  dwh->depconf.h_contract_terms = *h_contract_terms;
-  dwh->depconf.coin_pub = *coin_pub;
+  if (NULL == dwh->url)
+  {
+    GNUNET_free (dwh);
+    return NULL;
+  }
+  dwh->h_wire = *h_wire;
+  dwh->h_contract_terms = *h_contract_terms;
+  dwh->coin_pub = *coin_pub;
 
   eh = TALER_EXCHANGE_curl_easy_get_ (dwh->url);
   if (NULL == eh)
@@ -396,12 +358,6 @@ TALER_EXCHANGE_deposits_get (
 }
 
 
-/**
- * Cancel /deposits/$WTID request.  This function cannot be used on a request
- * handle if a response is already served for it.
- *
- * @param dwh the wire deposits request handle
- */
 void
 TALER_EXCHANGE_deposits_get_cancel (struct TALER_EXCHANGE_DepositGetHandle *dwh)
 {
