@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2017-2020 Taler Systems SA
+  Copyright (C) 2017-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -27,6 +27,7 @@
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_json_lib.h"
 #include "taler_exchange_service.h"
+#include "exchange_api_common.h"
 #include "exchange_api_handle.h"
 #include "taler_signatures.h"
 #include "exchange_api_curl_defaults.h"
@@ -59,6 +60,11 @@ struct TALER_EXCHANGE_RecoupHandle
    */
   struct TALER_EXCHANGE_DenomPublicKey pk;
 
+  /**
+   * Our signature requesting the recoup.
+   */
+  struct TALER_CoinSpendSignatureP coin_sig;
+
   /**
    * Handle for the request.
    */
@@ -79,11 +85,6 @@ struct TALER_EXCHANGE_RecoupHandle
    */
   struct TALER_CoinSpendPublicKeyP coin_pub;
 
-  /**
-   * #GNUNET_YES if the coin was refreshed
-   */
-  int was_refreshed;
-
 };
 
 
@@ -95,21 +96,14 @@ struct TALER_EXCHANGE_RecoupHandle
  * @return #GNUNET_OK if the signature is valid and we called the callback;
  *         #GNUNET_SYSERR if not (callback must still be called)
  */
-static int
+static enum GNUNET_GenericReturnValue
 process_recoup_response (const struct TALER_EXCHANGE_RecoupHandle *ph,
                          const json_t *json)
 {
-  int refreshed;
   struct TALER_ReservePublicKeyP reserve_pub;
-  struct TALER_CoinSpendPublicKeyP old_coin_pub;
   struct GNUNET_JSON_Specification spec_withdraw[] = {
-    GNUNET_JSON_spec_boolean ("refreshed", &refreshed),
-    GNUNET_JSON_spec_fixed_auto ("reserve_pub", &reserve_pub),
-    GNUNET_JSON_spec_end ()
-  };
-  struct GNUNET_JSON_Specification spec_refresh[] = {
-    GNUNET_JSON_spec_boolean ("refreshed", &refreshed),
-    GNUNET_JSON_spec_fixed_auto ("old_coin_pub", &old_coin_pub),
+    GNUNET_JSON_spec_fixed_auto ("reserve_pub",
+                                 &reserve_pub),
     GNUNET_JSON_spec_end ()
   };
   struct TALER_EXCHANGE_HttpResponse hr = {
@@ -119,21 +113,15 @@ process_recoup_response (const struct TALER_EXCHANGE_RecoupHandle *ph,
 
   if (GNUNET_OK !=
       GNUNET_JSON_parse (json,
-                         ph->was_refreshed ? spec_refresh : spec_withdraw,
+                         spec_withdraw,
                          NULL, NULL))
   {
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
-  if (ph->was_refreshed != refreshed)
-  {
-    GNUNET_break_op (0);
-    return GNUNET_SYSERR;
-  }
   ph->cb (ph->cb_cls,
           &hr,
-          ph->was_refreshed ? NULL : &reserve_pub,
-          ph->was_refreshed ? &old_coin_pub : NULL);
+          &reserve_pub);
   return GNUNET_OK;
 }
 
@@ -157,8 +145,10 @@ handle_recoup_finished (void *cls,
     .reply = j,
     .http_status = (unsigned int) response_code
   };
+  const struct TALER_EXCHANGE_Keys *keys;
 
   ph->job = NULL;
+  keys = TALER_EXCHANGE_get_keys (ph->exchange);
   switch (response_code)
   {
   case 0:
@@ -184,77 +174,34 @@ handle_recoup_finished (void *cls,
     break;
   case MHD_HTTP_CONFLICT:
     {
-      /* Insufficient funds, proof attached */
-      json_t *history;
-      struct TALER_Amount total;
-      struct GNUNET_HashCode h_denom_pub;
-      const struct TALER_EXCHANGE_DenomPublicKey *dki;
-      enum TALER_ErrorCode ec;
+      struct TALER_Amount min_key;
 
-      dki = &ph->pk;
-      history = json_object_get (j,
-                                 "history");
+      hr.ec = TALER_JSON_get_error_code (j);
+      hr.hint = TALER_JSON_get_error_hint (j);
       if (GNUNET_OK !=
-          TALER_EXCHANGE_verify_coin_history (dki,
-                                              dki->fee_deposit.currency,
-                                              &ph->coin_pub,
-                                              history,
-                                              &h_denom_pub,
-                                              &total))
+          TALER_EXCHANGE_get_min_denomination_ (keys,
+                                                &min_key))
       {
-        GNUNET_break_op (0);
-        hr.http_status = 0;
+        GNUNET_break (0);
         hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-      }
-      else
-      {
-        hr.ec = TALER_JSON_get_error_code (j);
-        hr.hint = TALER_JSON_get_error_hint (j);
-      }
-      ec = TALER_JSON_get_error_code (j);
-      switch (ec)
-      {
-      case TALER_EC_EXCHANGE_RECOUP_COIN_BALANCE_ZERO:
-        if (0 > TALER_amount_cmp (&total,
-                                  &dki->value))
-        {
-          /* recoup MAY have still been possible */
-          /* FIXME: This code may falsely complain, as we do not
-             know that the smallest denomination offered by the
-             exchange is here. We should look at the key
-             structure of ph->exchange, and find the smallest
-             _currently withdrawable_ denomination and check
-             if the value remaining would suffice... *///
-          GNUNET_break_op (0);
-          hr.http_status = 0;
-          hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-          break;
-        }
-        break;
-      case TALER_EC_EXCHANGE_GENERIC_COIN_CONFLICTING_DENOMINATION_KEY:
-        if (0 == GNUNET_memcmp (&ph->pk.h_key,
-                                &h_denom_pub))
-        {
-          /* invalid proof provided */
-          GNUNET_break_op (0);
-          hr.http_status = 0;
-          hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-          break;
-        }
-        /* valid error from exchange */
-        break;
-      default:
-        GNUNET_break_op (0);
         hr.http_status = 0;
-        hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
         break;
       }
-      ph->cb (ph->cb_cls,
-              &hr,
-              NULL,
-              NULL);
-      TALER_EXCHANGE_recoup_cancel (ph);
-      return;
+      if (GNUNET_OK !=
+          TALER_EXCHANGE_check_coin_conflict_ (
+            keys,
+            j,
+            &ph->pk,
+            &ph->coin_pub,
+            &ph->coin_sig,
+            &min_key))
+      {
+        GNUNET_break (0);
+        hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
+        hr.http_status = 0;
+        break;
+      }
+      break;
     }
   case MHD_HTTP_FORBIDDEN:
     /* Nothing really to verify, exchange says one of the signatures is
@@ -286,7 +233,7 @@ handle_recoup_finished (void *cls,
     hr.ec = TALER_JSON_get_error_code (j);
     hr.hint = TALER_JSON_get_error_hint (j);
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d\n",
+                "Unexpected response code %u/%d for exchange recoup\n",
                 (unsigned int) response_code,
                 (int) hr.ec);
     GNUNET_break (0);
@@ -294,86 +241,84 @@ handle_recoup_finished (void *cls,
   }
   ph->cb (ph->cb_cls,
           &hr,
-          NULL,
           NULL);
   TALER_EXCHANGE_recoup_cancel (ph);
 }
 
 
-/**
- * Ask the exchange to pay back a coin due to the exchange triggering
- * the emergency recoup protocol for a given denomination.  The value
- * of the coin will be refunded to the original customer (without fees).
- *
- * @param exchange the exchange handle; the exchange must be ready to operate
- * @param pk kind of coin to pay back
- * @param denom_sig signature over the coin by the exchange using @a pk
- * @param ps secret internals of the original planchet
- * @param was_refreshed #GNUNET_YES if the coin in @a ps was refreshed
- * @param recoup_cb the callback to call when the final result for this request is available
- * @param recoup_cb_cls closure for @a recoup_cb
- * @return NULL
- *         if the inputs are invalid (i.e. denomination key not with this exchange).
- *         In this case, the callback is not called.
- */
 struct TALER_EXCHANGE_RecoupHandle *
 TALER_EXCHANGE_recoup (struct TALER_EXCHANGE_Handle *exchange,
                        const struct TALER_EXCHANGE_DenomPublicKey *pk,
                        const struct TALER_DenominationSignature *denom_sig,
-                       const struct TALER_PlanchetSecretsP *ps,
-                       int was_refreshed,
+                       const struct TALER_ExchangeWithdrawValues *exchange_vals,
+                       const struct TALER_PlanchetMasterSecretP *ps,
                        TALER_EXCHANGE_RecoupResultCallback recoup_cb,
                        void *recoup_cb_cls)
 {
   struct TALER_EXCHANGE_RecoupHandle *ph;
   struct GNUNET_CURL_Context *ctx;
-  struct TALER_RecoupRequestPS pr;
-  struct TALER_CoinSpendSignatureP coin_sig;
-  struct GNUNET_HashCode h_denom_pub;
+  struct TALER_DenominationHashP h_denom_pub;
   json_t *recoup_obj;
   CURL *eh;
   char arg_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2 + 32];
+  struct TALER_CoinSpendPrivateKeyP coin_priv;
+  union TALER_DenominationBlindingKeyP bks;
 
   GNUNET_assert (GNUNET_YES ==
                  TEAH_handle_is_ready (exchange));
-  pr.purpose.purpose = htonl (TALER_SIGNATURE_WALLET_COIN_RECOUP);
-  pr.purpose.size = htonl (sizeof (struct TALER_RecoupRequestPS));
-  GNUNET_CRYPTO_eddsa_key_get_public (&ps->coin_priv.eddsa_priv,
-                                      &pr.coin_pub.eddsa_pub);
-  GNUNET_CRYPTO_rsa_public_key_hash (pk->key.rsa_public_key,
-                                     &h_denom_pub);
-  pr.h_denom_pub = pk->h_key;
-  pr.coin_blind = ps->blinding_key;
-  GNUNET_CRYPTO_eddsa_sign (&ps->coin_priv.eddsa_priv,
-                            &pr,
-                            &coin_sig.eddsa_signature);
-  recoup_obj = json_pack ("{s:o, s:o," /* denom pub/sig */
-                          " s:o,"  /* sig */
-                          " s:o, s:o}",  /* coin_bks */
-                          "denom_pub_hash", GNUNET_JSON_from_data_auto (
-                            &h_denom_pub),
-                          "denom_sig", GNUNET_JSON_from_rsa_signature (
-                            denom_sig->rsa_signature),
-                          "coin_sig", GNUNET_JSON_from_data_auto (&coin_sig),
-                          "coin_blind_key_secret", GNUNET_JSON_from_data_auto (
-                            &ps->blinding_key),
-                          "refreshed", json_boolean (was_refreshed)
-                          );
-  if (NULL == recoup_obj)
+  ph = GNUNET_new (struct TALER_EXCHANGE_RecoupHandle);
+  TALER_planchet_setup_coin_priv (ps,
+                                  exchange_vals,
+                                  &coin_priv);
+  TALER_planchet_blinding_secret_create (ps,
+                                         exchange_vals,
+                                         &bks);
+  GNUNET_CRYPTO_eddsa_key_get_public (&coin_priv.eddsa_priv,
+                                      &ph->coin_pub.eddsa_pub);
+  TALER_denom_pub_hash (&pk->key,
+                        &h_denom_pub);
+  TALER_wallet_recoup_sign (&h_denom_pub,
+                            &bks,
+                            &coin_priv,
+                            &ph->coin_sig);
+  recoup_obj = GNUNET_JSON_PACK (
+    GNUNET_JSON_pack_data_auto ("denom_pub_hash",
+                                &h_denom_pub),
+    TALER_JSON_pack_denom_sig ("denom_sig",
+                               denom_sig),
+    TALER_JSON_pack_exchange_withdraw_values ("ewv",
+                                              exchange_vals),
+    GNUNET_JSON_pack_data_auto ("coin_sig",
+                                &ph->coin_sig),
+    GNUNET_JSON_pack_data_auto ("coin_blind_key_secret",
+                                &bks));
+  if (TALER_DENOMINATION_CS == denom_sig->cipher)
   {
-    GNUNET_break (0);
-    return NULL;
+    struct TALER_CsNonce nonce;
+
+    /* NOTE: this is not elegant, and as per the note in TALER_coin_ev_hash()
+       it is not strictly clear that the nonce is needed. Best case would be
+       to find a way to include it more 'naturally' somehow, for example with
+       the variant union version of bks! */
+    TALER_cs_withdraw_nonce_derive (ps,
+                                    &nonce);
+    GNUNET_assert (
+      0 ==
+      json_object_set_new (recoup_obj,
+                           "cs_nonce",
+                           GNUNET_JSON_from_data_auto (
+                             &nonce)));
   }
 
   {
     char pub_str[sizeof (struct TALER_CoinSpendPublicKeyP) * 2];
     char *end;
 
-    end = GNUNET_STRINGS_data_to_string (&pr.coin_pub,
-                                         sizeof (struct
-                                                 TALER_CoinSpendPublicKeyP),
-                                         pub_str,
-                                         sizeof (pub_str));
+    end = GNUNET_STRINGS_data_to_string (
+      &ph->coin_pub,
+      sizeof (struct TALER_CoinSpendPublicKeyP),
+      pub_str,
+      sizeof (pub_str));
     *end = '\0';
     GNUNET_snprintf (arg_str,
                      sizeof (arg_str),
@@ -381,16 +326,21 @@ TALER_EXCHANGE_recoup (struct TALER_EXCHANGE_Handle *exchange,
                      pub_str);
   }
 
-  ph = GNUNET_new (struct TALER_EXCHANGE_RecoupHandle);
-  ph->coin_pub = pr.coin_pub;
   ph->exchange = exchange;
   ph->pk = *pk;
-  ph->pk.key.rsa_public_key = NULL; /* zero out, as lifetime cannot be warranted */
+  memset (&ph->pk.key,
+          0,
+          sizeof (ph->pk.key)); /* zero out, as lifetime cannot be warranted */
   ph->cb = recoup_cb;
   ph->cb_cls = recoup_cb_cls;
   ph->url = TEAH_path_to_url (exchange,
                               arg_str);
-  ph->was_refreshed = was_refreshed;
+  if (NULL == ph->url)
+  {
+    json_decref (recoup_obj);
+    GNUNET_free (ph);
+    return NULL;
+  }
   eh = TALER_EXCHANGE_curl_easy_get_ (ph->url);
   if ( (NULL == eh) ||
        (GNUNET_OK !=
@@ -420,12 +370,6 @@ TALER_EXCHANGE_recoup (struct TALER_EXCHANGE_Handle *exchange,
 }
 
 
-/**
- * Cancel a recoup request.  This function cannot be used on a
- * request handle if the callback was already invoked.
- *
- * @param ph the recoup handle
- */
 void
 TALER_EXCHANGE_recoup_cancel (struct TALER_EXCHANGE_RecoupHandle *ph)
 {
