@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2018 Taler Systems SA
+  Copyright (C) 2014-2020 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -29,6 +29,7 @@
 #include "taler_auditor_service.h"
 #include "taler_json_lib.h"
 #include <gnunet/gnunet_util_lib.h>
+#include <gnunet/gnunet_testing_lib.h>
 #include <microhttpd.h>
 #include "taler_bank_service.h"
 #include "taler_fakebank_lib.h"
@@ -39,10 +40,9 @@
  * Configuration file we use.  One (big) configuration is used
  * for the various components for this test.
  */
-#define CONFIG_FILE "test_auditor_api.conf"
+static char *config_file;
 
-#define CONFIG_FILE_EXPIRE_RESERVE_NOW \
-  "test_auditor_api_expire_reserve_now.conf"
+static char *config_file_expire_reserve_now;
 
 /**
  * Exchange configuration data.
@@ -61,7 +61,7 @@ static struct TALER_TESTING_BankConfiguration bc;
  * @param label label to use for the command.
  */
 #define CMD_EXEC_WIREWATCH(label) \
-  TALER_TESTING_cmd_exec_wirewatch (label, CONFIG_FILE)
+  TALER_TESTING_cmd_exec_wirewatch (label, config_file)
 
 /**
  * Execute the taler-exchange-aggregator, closer and transfer commands with
@@ -70,8 +70,9 @@ static struct TALER_TESTING_BankConfiguration bc;
  * @param label label to use for the command.
  */
 #define CMD_EXEC_AGGREGATOR(label) \
-  TALER_TESTING_cmd_exec_aggregator (label, CONFIG_FILE), \
-  TALER_TESTING_cmd_exec_transfer (label, CONFIG_FILE)
+  TALER_TESTING_cmd_sleep (label "-sleep", 1), \
+  TALER_TESTING_cmd_exec_aggregator (label, config_file), \
+  TALER_TESTING_cmd_exec_transfer (label, config_file)
 
 /**
  * Run wire transfer of funds from some user's account to the
@@ -91,7 +92,7 @@ static struct TALER_TESTING_BankConfiguration bc;
  * @param label label to use for the command.
  */
 #define CMD_RUN_AUDITOR(label) \
-  TALER_TESTING_cmd_exec_auditor (label, CONFIG_FILE)
+  TALER_TESTING_cmd_exec_auditor (label, config_file)
 
 
 /**
@@ -127,6 +128,7 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-1",
                                        "create-reserve-1",
                                        "EUR:5",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_end ()
   };
@@ -167,6 +169,7 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("refresh-withdraw-coin-1",
                                        "refresh-create-reserve-1",
                                        "EUR:5",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     /**
      * Try to partially spend (deposit) 1 EUR of the 5 EUR coin (in
@@ -182,7 +185,8 @@ run (void *cls,
                                MHD_HTTP_OK),
     /**
      * Melt the rest of the coin's value (EUR:4.00 = 3x EUR:1.03 + 7x
-     * EUR:0.13) */
+     * EUR:0.13)
+     */
     TALER_TESTING_cmd_melt_double ("refresh-melt-1",
                                    "refresh-withdraw-coin-1",
                                    MHD_HTTP_OK,
@@ -213,26 +217,35 @@ run (void *cls,
      * happen here, as each deposit operation is run with a
      * fresh merchant public key! NOTE: this comment comes
      * "verbatim" from the old test-suite, and IMO does not explain
-     * a lot!*///
+     * a lot! */
     CMD_EXEC_AGGREGATOR ("run-aggregator"),
 
     /**
      * Check all the transfers took place.
      */
-    TALER_TESTING_cmd_check_bank_transfer
-      ("check_bank_transfer-499c", ec.exchange_url,
-      "EUR:4.98", bc.exchange_payto, bc.user42_payto),
-    TALER_TESTING_cmd_check_bank_transfer
-      ("check_bank_transfer-99c1", ec.exchange_url,
-      "EUR:0.98", bc.exchange_payto, bc.user42_payto),
-    TALER_TESTING_cmd_check_bank_transfer
-      ("check_bank_transfer-99c", ec.exchange_url,
-      "EUR:0.08", bc.exchange_payto, bc.user43_payto),
+    TALER_TESTING_cmd_check_bank_transfer (
+      "check_bank_transfer-499c",
+      ec.exchange_url,
+      "EUR:4.98",
+      bc.exchange_payto,
+      bc.user42_payto),
+    TALER_TESTING_cmd_check_bank_transfer (
+      "check_bank_transfer-99c1",
+      ec.exchange_url,
+      "EUR:0.98",
+      bc.exchange_payto,
+      bc.user42_payto),
+    TALER_TESTING_cmd_check_bank_transfer (
+      "check_bank_transfer-99c",
+      ec.exchange_url,
+      "EUR:0.08",
+      bc.exchange_payto,
+      bc.user43_payto),
 
     /* The following transactions got originated within
      * the "massive deposit confirms" batch.  */
-    TALER_TESTING_cmd_check_bank_transfer
-      ("check-massive-transfer-1",
+    TALER_TESTING_cmd_check_bank_transfer (
+      "check-massive-transfer-1",
       ec.exchange_url,
       "EUR:0.98",
       bc.exchange_payto, bc.user43_payto),
@@ -304,6 +317,7 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-unaggregated",
                                        "create-reserve-unaggregated",
                                        "EUR:5",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_deposit ("deposit-unaggregated",
                                "withdraw-coin-unaggregated",
@@ -336,6 +350,7 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("withdraw-coin-r1",
                                        "create-reserve-r1",
                                        "EUR:5",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     /**
      * Spend 5 EUR of the 5 EUR coin (in full). Merchant would
@@ -391,15 +406,15 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("recoup-withdraw-coin-1",
                                        "recoup-create-reserve-1",
                                        "EUR:5",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_revoke ("revoke-1",
                               MHD_HTTP_OK,
                               "recoup-withdraw-coin-1",
-                              CONFIG_FILE),
+                              config_file),
     TALER_TESTING_cmd_recoup ("recoup-1",
                               MHD_HTTP_OK,
                               "recoup-withdraw-coin-1",
-                              NULL,
                               "EUR:5"),
     /**
      * Re-withdraw from this reserve
@@ -407,17 +422,19 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("recoup-withdraw-coin-2",
                                        "recoup-create-reserve-1",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     /**
      * These commands should close the reserve because the aggregator
      * is given a config file that overrides the reserve expiration
      * time (making it now-ish)
-     */CMD_TRANSFER_TO_EXCHANGE ("short-lived-reserve",
+     */
+    CMD_TRANSFER_TO_EXCHANGE ("short-lived-reserve",
                               "EUR:5.01"),
     TALER_TESTING_cmd_exec_wirewatch ("short-lived-aggregation",
-                                      CONFIG_FILE_EXPIRE_RESERVE_NOW),
+                                      config_file_expire_reserve_now),
     TALER_TESTING_cmd_exec_aggregator ("close-reserves",
-                                       CONFIG_FILE_EXPIRE_RESERVE_NOW),
+                                       config_file_expire_reserve_now),
     /**
      * Fill reserve with EUR:2.02, as withdraw fee is 1 ct per
      * config, then withdraw two coin, partially spend one, and
@@ -436,6 +453,7 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("recoup-withdraw-coin-2a",
                                        "recoup-create-reserve-2",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     /**
      * Withdraw a 1 EUR coin, at fee of 1 ct
@@ -443,6 +461,7 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("recoup-withdraw-coin-2b",
                                        "recoup-create-reserve-2",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_deposit ("recoup-deposit-partial",
                                "recoup-withdraw-coin-2a",
@@ -455,11 +474,10 @@ run (void *cls,
     TALER_TESTING_cmd_revoke ("revoke-2",
                               MHD_HTTP_OK,
                               "recoup-withdraw-coin-2a",
-                              CONFIG_FILE),
+                              config_file),
     TALER_TESTING_cmd_recoup ("recoup-2",
                               MHD_HTTP_OK,
                               "recoup-withdraw-coin-2a",
-                              NULL,
                               "EUR:0.5"),
     TALER_TESTING_cmd_end ()
   };
@@ -472,8 +490,8 @@ run (void *cls,
      */
     CMD_TRANSFER_TO_EXCHANGE ("massive-reserve",
                               "EUR:10.10"),
-    TALER_TESTING_cmd_check_bank_admin_transfer
-      ("check-massive-transfer",
+    TALER_TESTING_cmd_check_bank_admin_transfer (
+      "check-massive-transfer",
       "EUR:10.10",
       bc.user42_payto, bc.exchange_payto,
       "massive-reserve"),
@@ -481,45 +499,55 @@ run (void *cls,
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-1",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-2",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-3",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-4",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-5",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-6",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-7",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-8",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-9",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
     TALER_TESTING_cmd_withdraw_amount ("massive-withdraw-10",
                                        "massive-reserve",
                                        "EUR:1",
+                                       0, /* age restriction off */
                                        MHD_HTTP_OK),
-    TALER_TESTING_cmd_deposit
-      ("massive-deposit-1",
+    TALER_TESTING_cmd_deposit (
+      "massive-deposit-1",
       "massive-withdraw-1",
       0,
       bc.user43_payto,
@@ -599,8 +627,8 @@ run (void *cls,
       GNUNET_TIME_UNIT_ZERO,
       "EUR:1",
       MHD_HTTP_OK),
-    TALER_TESTING_cmd_deposit
-      ("massive-deposit-10",
+    TALER_TESTING_cmd_deposit (
+      "massive-deposit-10",
       "massive-withdraw-10",
       0,
       bc.user43_payto,
@@ -620,6 +648,23 @@ run (void *cls,
   };
 
   struct TALER_TESTING_Command commands[] = {
+    TALER_TESTING_cmd_exec_offline_sign_fees ("offline-sign-fees",
+                                              config_file,
+                                              "EUR:0.01",
+                                              "EUR:0.01"),
+    TALER_TESTING_cmd_auditor_add ("add-auditor-OK",
+                                   MHD_HTTP_NO_CONTENT,
+                                   false),
+    TALER_TESTING_cmd_wire_add ("add-wire-account",
+                                "payto://x-taler-bank/localhost/2?receiver-name=2",
+                                MHD_HTTP_NO_CONTENT,
+                                false),
+    TALER_TESTING_cmd_exec_offline_sign_keys ("offline-sign-future-keys",
+                                              config_file),
+    TALER_TESTING_cmd_check_keys_pull_all_keys ("refetch /keys",
+                                                2),
+    TALER_TESTING_cmd_exec_auditor_offline ("auditor-offline",
+                                            config_file),
     CMD_RUN_AUDITOR ("virgin-auditor"),
     TALER_TESTING_cmd_exchanges_with_url ("check-exchange",
                                           MHD_HTTP_OK,
@@ -644,6 +689,7 @@ run (void *cls,
     TALER_TESTING_cmd_end ()
   };
 
+  (void) cls;
   TALER_TESTING_run_with_fakebank (is,
                                    commands,
                                    bc.exchange_auth.wire_gateway_url);
@@ -654,23 +700,36 @@ int
 main (int argc,
       char *const *argv)
 {
+  char *cipher;
+
+  (void) argc;
   /* These environment variables get in the way... */
   unsetenv ("XDG_DATA_HOME");
   unsetenv ("XDG_CONFIG_HOME");
-  GNUNET_log_setup ("test-auditor-api",
+  GNUNET_log_setup (argv[0],
                     "INFO",
                     NULL);
+
+  cipher = GNUNET_TESTING_get_testname_from_underscore (argv[0]);
+  GNUNET_assert (NULL != cipher);
+  GNUNET_asprintf (&config_file,
+                   "test_auditor_api-%s.conf",
+                   cipher);
+  GNUNET_asprintf (&config_file_expire_reserve_now,
+                   "test_auditor_api_expire_reserve_now-%s.conf",
+                   cipher);
+  GNUNET_free (cipher);
   /* Check fakebank port is available and get configuration data. */
   if (GNUNET_OK !=
-      TALER_TESTING_prepare_fakebank (CONFIG_FILE,
+      TALER_TESTING_prepare_fakebank (config_file,
                                       "exchange-account-2",
                                       &bc))
     return 77;
-  TALER_TESTING_cleanup_files (CONFIG_FILE);
+  TALER_TESTING_cleanup_files (config_file);
   /* @helpers.  Run keyup, create tables, ... Note: it
    * fetches the port number from config in order to see
    * if it's available. */
-  switch (TALER_TESTING_prepare_exchange (CONFIG_FILE,
+  switch (TALER_TESTING_prepare_exchange (config_file,
                                           GNUNET_YES,
                                           &ec))
   {
@@ -678,7 +737,7 @@ main (int argc,
     GNUNET_break (0);
     return 1;
   case GNUNET_NO:
-    return 77;
+    return 78;
   case GNUNET_OK:
     if (GNUNET_OK !=
         /* Set up event loop and reschedule context, plus
@@ -687,12 +746,12 @@ main (int argc,
          */
         TALER_TESTING_auditor_setup (&run,
                                      NULL,
-                                     CONFIG_FILE))
-      return 1;
+                                     config_file))
+      return 2;
     break;
   default:
     GNUNET_break (0);
-    return 1;
+    return 3;
   }
   return 0;
 }
