@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2018-2020 Taler Systems SA
+  Copyright (C) 2018-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
@@ -107,22 +107,17 @@ struct AdminAddIncomingState
    * the "sender_url" field is set to a 'const char *' and
    * MUST NOT be free()'ed.
    */
-  struct TALER_EXCHANGE_ReserveHistory reserve_history;
+  struct TALER_EXCHANGE_ReserveHistoryEntry reserve_history;
 
   /**
    * Set to the wire transfer's unique ID.
    */
   uint64_t serial_id;
 
-  /**
-   * Set to the wire transfer's row ID in network byte order.
-   */
-  uint64_t row_id_nbo;
-
   /**
    * Timestamp of the transaction (as returned from the bank).
    */
-  struct GNUNET_TIME_Absolute timestamp;
+  struct GNUNET_TIME_Timestamp timestamp;
 
   /**
    * Merchant instance.  Sometimes used to get the tip reserve
@@ -153,6 +148,11 @@ struct AdminAddIncomingState
    * enable retries? If so, how often should we still retry?
    */
   unsigned int do_retry;
+
+  /**
+   * Expected HTTP status code.
+   */
+  unsigned int expected_http_status;
 };
 
 
@@ -207,19 +207,42 @@ confirmation_cb (void *cls,
                  unsigned int http_status,
                  enum TALER_ErrorCode ec,
                  uint64_t serial_id,
-                 struct GNUNET_TIME_Absolute timestamp,
+                 struct GNUNET_TIME_Timestamp timestamp,
                  const json_t *json)
 {
   struct AdminAddIncomingState *fts = cls;
   struct TALER_TESTING_Interpreter *is = fts->is;
 
   (void) json;
-  fts->row_id_nbo = GNUNET_htonll (serial_id);
   fts->reserve_history.details.in_details.timestamp = timestamp;
-  fts->reserve_history.details.in_details.wire_reference = &fts->row_id_nbo;
-  fts->reserve_history.details.in_details.wire_reference_size
-    = sizeof (fts->row_id_nbo);
+  fts->reserve_history.details.in_details.wire_reference = serial_id;
   fts->aih = NULL;
+  /**
+   * Test case not caring about the HTTP status code.
+   * That helps when Fakebank and Libeufin diverge in
+   * the response status code.  An example is the
+   * /admin/add-incoming: libeufin return ALWAYS '200 OK'
+   * (see note below) whereas the Fakebank responds with
+   * '409 Conflict' upon a duplicate reserve public key.
+   *
+   * Note: this decision aims at avoiding to put Taler
+   * logic into the Sandbox; that's because banks DO allow
+   * their customers to wire the same subject multiple
+   * times.  Hence, instead of triggering any error, libeufin
+   * bounces the payment back in the same way it does for
+   * malformed reserve public keys.
+   */
+  if (-1 == (int) fts->expected_http_status)
+  {
+    TALER_TESTING_interpreter_next (is);
+    return;
+  }
+  if (http_status != fts->expected_http_status)
+  {
+    GNUNET_break (0);
+    TALER_TESTING_interpreter_fail (is);
+    return;
+  }
   switch (http_status)
   {
   case MHD_HTTP_OK:
@@ -241,6 +264,9 @@ confirmation_cb (void *cls,
       break;
     }
     break;
+  case MHD_HTTP_CONFLICT:
+    TALER_TESTING_interpreter_next (is);
+    return;
   default:
     if (0 != fts->do_retry)
     {
@@ -312,11 +338,9 @@ admin_add_incoming_run (void *cls,
     }
     if (GNUNET_OK !=
         TALER_TESTING_get_trait_reserve_priv (ref,
-                                              0,
                                               &reserve_priv))
     {
       if (GNUNET_OK != TALER_TESTING_get_trait_reserve_pub (ref,
-                                                            0,
                                                             &reserve_pub))
       {
         GNUNET_break (0);
@@ -406,30 +430,33 @@ admin_add_incoming_cleanup (void *cls,
  * @param index index number of the object to offer.
  * @return #GNUNET_OK on success.
  */
-static int
+static enum GNUNET_GenericReturnValue
 admin_add_incoming_traits (void *cls,
                            const void **ret,
                            const char *trait,
                            unsigned int index)
 {
   struct AdminAddIncomingState *fts = cls;
+  static const char *void_uri = "payto://void/the-exchange";
+
+  if (MHD_HTTP_OK !=
+      fts->expected_http_status)
+    return GNUNET_NO; /* requests that failed generate no history */
   if (fts->reserve_priv_known)
   {
     struct TALER_TESTING_Trait traits[] = {
       TALER_TESTING_make_trait_bank_row (&fts->serial_id),
-      TALER_TESTING_make_trait_payto (TALER_TESTING_PT_DEBIT,
-                                      fts->payto_debit_account),
+      TALER_TESTING_make_trait_debit_payto_uri (&fts->payto_debit_account),
+      TALER_TESTING_make_trait_payto_uri (&fts->payto_debit_account),
       /* Used as a marker, content does not matter */
-      TALER_TESTING_make_trait_payto (TALER_TESTING_PT_CREDIT,
-                                      "payto://void/the-exchange"),
-      TALER_TESTING_make_trait_url (TALER_TESTING_UT_EXCHANGE_BANK_ACCOUNT_URL,
-                                    fts->exchange_credit_url),
-      TALER_TESTING_make_trait_amount_obj (0, &fts->amount),
-      TALER_TESTING_make_trait_absolute_time (0, &fts->timestamp),
-      TALER_TESTING_make_trait_reserve_priv (0,
-                                             &fts->reserve_priv),
-      TALER_TESTING_make_trait_reserve_pub (0,
-                                            &fts->reserve_pub),
+      TALER_TESTING_make_trait_credit_payto_uri (&void_uri),
+      TALER_TESTING_make_trait_exchange_bank_account_url (
+        &fts->exchange_credit_url),
+      TALER_TESTING_make_trait_amount (&fts->amount),
+      TALER_TESTING_make_trait_timestamp (0,
+                                          &fts->timestamp),
+      TALER_TESTING_make_trait_reserve_priv (&fts->reserve_priv),
+      TALER_TESTING_make_trait_reserve_pub (&fts->reserve_pub),
       TALER_TESTING_make_trait_reserve_history (0,
                                                 &fts->reserve_history),
       TALER_TESTING_trait_end ()
@@ -444,17 +471,14 @@ admin_add_incoming_traits (void *cls,
   {
     struct TALER_TESTING_Trait traits[] = {
       TALER_TESTING_make_trait_bank_row (&fts->serial_id),
-      TALER_TESTING_make_trait_payto (TALER_TESTING_PT_DEBIT,
-                                      fts->payto_debit_account),
+      TALER_TESTING_make_trait_debit_payto_uri (&fts->payto_debit_account),
       /* Used as a marker, content does not matter */
-      TALER_TESTING_make_trait_payto (TALER_TESTING_PT_CREDIT,
-                                      "payto://void/the-exchange"),
-      TALER_TESTING_make_trait_url (TALER_TESTING_UT_EXCHANGE_BANK_ACCOUNT_URL,
-                                    fts->exchange_credit_url),
-      TALER_TESTING_make_trait_amount_obj (0, &fts->amount),
-      TALER_TESTING_make_trait_absolute_time (0, &fts->timestamp),
-      TALER_TESTING_make_trait_reserve_pub (0,
-                                            &fts->reserve_pub),
+      TALER_TESTING_make_trait_credit_payto_uri (&void_uri),
+      TALER_TESTING_make_trait_exchange_bank_account_url (
+        &fts->exchange_credit_url),
+      TALER_TESTING_make_trait_amount (&fts->amount),
+      TALER_TESTING_make_trait_timestamp (0, &fts->timestamp),
+      TALER_TESTING_make_trait_reserve_pub (&fts->reserve_pub),
       TALER_TESTING_make_trait_reserve_history (0,
                                                 &fts->reserve_history),
       TALER_TESTING_trait_end ()
@@ -487,6 +511,7 @@ make_fts (const char *amount,
   fts->exchange_credit_url = auth->wire_gateway_url;
   fts->payto_debit_account = payto_debit_account;
   fts->auth = *auth;
+  fts->expected_http_status = MHD_HTTP_OK;
   if (GNUNET_OK !=
       TALER_string_to_amount (amount,
                               &fts->amount))
@@ -523,21 +548,12 @@ make_command (const char *label,
 }
 
 
-/**
- * Create admin/add-incoming command.
- *
- * @param label command label.
- * @param amount amount to transfer.
- * @param payto_debit_account which account sends money.
- * @param auth authentication data
- * @return the command.
- */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_admin_add_incoming (const char *label,
-                                      const char *amount,
-                                      const struct
-                                      TALER_BANK_AuthenticationData *auth,
-                                      const char *payto_debit_account)
+TALER_TESTING_cmd_admin_add_incoming (
+  const char *label,
+  const char *amount,
+  const struct TALER_BANK_AuthenticationData *auth,
+  const char *payto_debit_account)
 {
   return make_command (label,
                        make_fts (amount,
@@ -546,27 +562,14 @@ TALER_TESTING_cmd_admin_add_incoming (const char *label,
 }
 
 
-/**
- * Create "/admin/add-incoming" CMD, letting the caller specify
- * a reference to a command that can offer a reserve private key.
- * This private key will then be used to construct the subject line
- * of the wire transfer.
- *
- * @param label command label.
- * @param amount the amount to transfer.
- * @param payto_debit_account which account sends money
- * @param auth authentication data
- * @param ref reference to a command that can offer a reserve
- *        private key or public key.
- * @return the command.
- */
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_admin_add_incoming_with_ref
-  (const char *label,
+TALER_TESTING_cmd_admin_add_incoming_with_ref (
+  const char *label,
   const char *amount,
   const struct TALER_BANK_AuthenticationData *auth,
   const char *payto_debit_account,
-  const char *ref)
+  const char *ref,
+  unsigned int http_status)
 {
   struct AdminAddIncomingState *fts;
 
@@ -574,6 +577,7 @@ TALER_TESTING_cmd_admin_add_incoming_with_ref
                   auth,
                   payto_debit_account);
   fts->reserve_reference = ref;
+  fts->expected_http_status = http_status;
   return make_command (label,
                        fts);
 }
