@@ -553,8 +553,8 @@ const char *
 TALER_amount2s (const struct TALER_Amount *amount)
 {
   /* 24 is sufficient for a uint64_t value in decimal; 3 is for ":.\0" */
-  static GNUNET_THREAD_LOCAL char result[TALER_AMOUNT_FRAC_LEN
-                                         + TALER_CURRENCY_LEN + 3 + 24];
+  static TALER_THREAD_LOCAL char result[TALER_AMOUNT_FRAC_LEN
+                                        + TALER_CURRENCY_LEN + 3 + 24];
   struct TALER_Amount norm;
 
   if (GNUNET_YES != TALER_amount_is_valid (amount))
