@@ -766,6 +766,9 @@ handle_batch_sign_request (struct TES_Client *client,
     off += s;
     size -= s;
   }
+  GNUNET_break_op (0 == size);
+  bs = GNUNET_MIN (bs,
+                   idx);
   for (unsigned int i = 0; i<bs; i++)
     finish_job (client,
                 &jobs[i]);
@@ -1568,7 +1571,9 @@ parse_key (struct Denomination *denom,
          NULL != pos;
          pos = pos->next)
     {
-      if (GNUNET_TIME_timestamp_cmp (pos->anchor, >, anchor))
+      if (GNUNET_TIME_timestamp_cmp (pos->anchor,
+                                     >,
+                                     anchor))
         break;
       before = pos;
     }
@@ -1981,7 +1986,7 @@ run (void *cls,
   GNUNET_SCHEDULER_add_shutdown (&do_shutdown,
                                  NULL);
   if (0 == max_workers)
-    max_workers = 1; /* FIXME-#7272: or determine from CPU? */
+    max_workers = sysconf (_SC_NPROCESSORS_CONF);
   for (unsigned int i = 0; i<max_workers; i++)
     if (GNUNET_OK !=
         start_worker ())
