@@ -289,11 +289,18 @@ test_r_derive (struct TALER_CRYPTO_CsDenominationHelper *dh)
                 "Requesting R derivation with key %s\n",
                 GNUNET_h2s (&keys[i].h_cs.hash));
     alg_values.cipher = TALER_DENOMINATION_CS;
-    ec = TALER_CRYPTO_helper_cs_r_derive_withdraw (
-      dh,
-      &keys[i].h_cs,
-      &pd.blinded_planchet.details.cs_blinded_planchet.nonce,
-      &alg_values.details.cs_values);
+    {
+      struct TALER_CRYPTO_CsDeriveRequest cdr = {
+        .h_cs = &keys[i].h_cs,
+        .nonce = &pd.blinded_planchet.details.cs_blinded_planchet.nonce
+      };
+
+      ec = TALER_CRYPTO_helper_cs_r_derive (
+        dh,
+        &cdr,
+        false,
+        &alg_values.details.cs_values);
+    }
     switch (ec)
     {
     case TALER_EC_NONE:
@@ -374,6 +381,10 @@ test_r_derive (struct TALER_CRYPTO_CsDenominationHelper *dh)
     struct TALER_CsPubHashP rnd;
     struct TALER_CsNonce nonce;
     struct TALER_DenominationCSPublicRPairP crp;
+    struct TALER_CRYPTO_CsDeriveRequest cdr = {
+      .h_cs = &rnd,
+      .nonce = &nonce,
+    };
 
     GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                                 &rnd,
@@ -381,10 +392,10 @@ test_r_derive (struct TALER_CRYPTO_CsDenominationHelper *dh)
     GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                                 &nonce,
                                 sizeof (nonce));
-    ec = TALER_CRYPTO_helper_cs_r_derive_withdraw (dh,
-                                                   &rnd,
-                                                   &nonce,
-                                                   &crp);
+    ec = TALER_CRYPTO_helper_cs_r_derive (dh,
+                                          &cdr,
+                                          false,
+                                          &crp);
     if (TALER_EC_EXCHANGE_GENERIC_DENOMINATION_KEY_UNKNOWN != ec)
     {
       GNUNET_break (0);
@@ -423,20 +434,21 @@ test_signing (struct TALER_CRYPTO_CsDenominationHelper *dh)
       continue;
     {
       struct TALER_PlanchetDetail pd;
+      struct TALER_CRYPTO_CsSignRequest csr;
+      struct TALER_CRYPTO_CsDeriveRequest cdr = {
+        .h_cs = &keys[i].h_cs,
+        .nonce = &pd.blinded_planchet.details.cs_blinded_planchet.nonce
+      };
 
       pd.blinded_planchet.cipher = TALER_DENOMINATION_CS;
-      // keys[i].denom_pub.cipher = TALER_DENOMINATION_CS;
-
       TALER_cs_withdraw_nonce_derive (&ps,
                                       &pd.blinded_planchet.details.
                                       cs_blinded_planchet.nonce);
       alg_values.cipher = TALER_DENOMINATION_CS;
-      ec = TALER_CRYPTO_helper_cs_r_derive_withdraw (
+      ec = TALER_CRYPTO_helper_cs_r_derive (
         dh,
-        &keys[i].h_cs,
-        &pd.blinded_planchet.
-        details.
-        cs_blinded_planchet.nonce,
+        &cdr,
+        false,
         &alg_values.details.cs_values);
       if (TALER_EC_NONE != ec)
         continue;
@@ -458,11 +470,13 @@ test_signing (struct TALER_CRYPTO_CsDenominationHelper *dh)
       GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                   "Requesting signature with key %s\n",
                   GNUNET_h2s (&keys[i].h_cs.hash));
-      ec = TALER_CRYPTO_helper_cs_sign_withdraw (
+      csr.h_cs = &keys[i].h_cs;
+      csr.blinded_planchet
+        = &pd.blinded_planchet.details.cs_blinded_planchet;
+      ec = TALER_CRYPTO_helper_cs_sign (
         dh,
-        &keys[i].h_cs,
-        &pd.blinded_planchet.details.
-        cs_blinded_planchet,
+        &csr,
+        false,
         &ds);
     }
     switch (ec)
@@ -544,6 +558,7 @@ test_signing (struct TALER_CRYPTO_CsDenominationHelper *dh)
   {
     struct TALER_PlanchetDetail pd;
     struct TALER_CsPubHashP rnd;
+    struct TALER_CRYPTO_CsSignRequest csr;
 
     GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
                                 &rnd,
@@ -557,11 +572,13 @@ test_signing (struct TALER_CRYPTO_CsDenominationHelper *dh)
                                            NULL, /* no age commitment */
                                            &c_hash,
                                            &pd));
-
-    ec = TALER_CRYPTO_helper_cs_sign_withdraw (
+    csr.h_cs = &rnd;
+    csr.blinded_planchet
+      = &pd.blinded_planchet.details.cs_blinded_planchet;
+    ec = TALER_CRYPTO_helper_cs_sign (
       dh,
-      &rnd,
-      &pd.blinded_planchet.details.cs_blinded_planchet,
+      &csr,
+      false,
       &ds);
     if (TALER_EC_EXCHANGE_GENERIC_DENOMINATION_KEY_UNKNOWN != ec)
     {
@@ -578,6 +595,212 @@ test_signing (struct TALER_CRYPTO_CsDenominationHelper *dh)
 }
 
 
+/**
+ * Test batch signing logic.
+ *
+ * @param dh handle to the helper
+ * @param batch_size how large should the batch be
+ * @param check_sigs also check unknown key and signatures
+ * @return 0 on success
+ */
+static int
+test_batch_signing (struct TALER_CRYPTO_CsDenominationHelper *dh,
+                    unsigned int batch_size,
+                    bool check_sigs)
+{
+  struct TALER_BlindedDenominationSignature ds[batch_size];
+  enum TALER_ErrorCode ec;
+  bool success = false;
+  struct TALER_PlanchetMasterSecretP ps[batch_size];
+  struct TALER_CoinSpendPrivateKeyP coin_priv[batch_size];
+  union TALER_DenominationBlindingKeyP bks[batch_size];
+  struct TALER_CoinPubHashP c_hash[batch_size];
+  struct TALER_ExchangeWithdrawValues alg_values[batch_size];
+
+  for (unsigned int i = 0; i<batch_size; i++)
+    TALER_planchet_master_setup_random (&ps[i]);
+  for (unsigned int k = 0; k<MAX_KEYS; k++)
+  {
+    if (! keys[k].valid)
+      continue;
+    {
+      struct TALER_PlanchetDetail pd[batch_size];
+      struct TALER_CRYPTO_CsSignRequest csr[batch_size];
+      struct TALER_CRYPTO_CsDeriveRequest cdr[batch_size];
+      struct TALER_DenominationCSPublicRPairP crps[batch_size];
+
+      for (unsigned int i = 0; i<batch_size; i++)
+      {
+        cdr[i].h_cs = &keys[k].h_cs;
+        cdr[i].nonce =
+          &pd[i].blinded_planchet.details.cs_blinded_planchet.nonce;
+        pd[i].blinded_planchet.cipher = TALER_DENOMINATION_CS;
+        TALER_cs_withdraw_nonce_derive (
+          &ps[i],
+          &pd[i].blinded_planchet.details.cs_blinded_planchet.nonce);
+        alg_values[i].cipher = TALER_DENOMINATION_CS;
+      }
+      ec = TALER_CRYPTO_helper_cs_r_batch_derive (
+        dh,
+        cdr,
+        batch_size,
+        false,
+        crps);
+      if (TALER_EC_NONE != ec)
+        continue;
+      for (unsigned int i = 0; i<batch_size; i++)
+      {
+        alg_values[i].details.cs_values = crps[i];
+        TALER_planchet_setup_coin_priv (&ps[i],
+                                        &alg_values[i],
+                                        &coin_priv[i]);
+        TALER_planchet_blinding_secret_create (&ps[i],
+                                               &alg_values[i],
+                                               &bks[i]);
+        GNUNET_assert (GNUNET_YES ==
+                       TALER_planchet_prepare (&keys[k].denom_pub,
+                                               &alg_values[i],
+                                               &bks[i],
+                                               &coin_priv[i],
+                                               NULL, /* no age commitment */
+                                               &c_hash[i],
+                                               &pd[i]));
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Requesting signature with key %s\n",
+                    GNUNET_h2s (&keys[k].h_cs.hash));
+        csr[i].h_cs = &keys[k].h_cs;
+        csr[i].blinded_planchet
+          = &pd[i].blinded_planchet.details.cs_blinded_planchet;
+      }
+      ec = TALER_CRYPTO_helper_cs_batch_sign (
+        dh,
+        csr,
+        batch_size,
+        false,
+        ds);
+    }
+    switch (ec)
+    {
+    case TALER_EC_NONE:
+      if (GNUNET_TIME_relative_cmp (GNUNET_TIME_absolute_get_remaining (
+                                      keys[k].start_time.abs_time),
+                                    >,
+                                    GNUNET_TIME_UNIT_SECONDS))
+      {
+        /* key worked too early */
+        GNUNET_break (0);
+        return 4;
+      }
+      if (GNUNET_TIME_relative_cmp (GNUNET_TIME_absolute_get_duration (
+                                      keys[k].start_time.abs_time),
+                                    >,
+                                    keys[k].validity_duration))
+      {
+        /* key worked too later */
+        GNUNET_break (0);
+        return 5;
+      }
+      if (check_sigs)
+      {
+        for (unsigned int i = 0; i<batch_size; i++)
+        {
+          struct TALER_FreshCoin coin;
+
+          if (GNUNET_OK !=
+              TALER_planchet_to_coin (&keys[k].denom_pub,
+                                      &ds[i],
+                                      &bks[i],
+                                      &coin_priv[i],
+                                      NULL, /* no age commitment */
+                                      &c_hash[i],
+                                      &alg_values[i],
+                                      &coin))
+          {
+            GNUNET_break (0);
+            return 6;
+          }
+        }
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Received valid signature for key %s\n",
+                    GNUNET_h2s (&keys[k].h_cs.hash));
+      }
+      success = true;
+      break;
+    case TALER_EC_EXCHANGE_DENOMINATION_HELPER_TOO_EARLY:
+      /* This 'failure' is expected, we're testing also for the
+         error handling! */
+      if ( (GNUNET_TIME_relative_is_zero (
+              GNUNET_TIME_absolute_get_remaining (
+                keys[k].start_time.abs_time))) &&
+           (GNUNET_TIME_relative_cmp (
+              GNUNET_TIME_absolute_get_duration (
+                keys[k].start_time.abs_time),
+              <,
+              keys[k].validity_duration)) )
+      {
+        /* key should have worked! */
+        GNUNET_break (0);
+        return 6;
+      }
+      break;
+    default:
+      /* unexpected error */
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Unexpected error %d\n",
+                  ec);
+      return 7;
+    }
+  }
+  if (! success)
+  {
+    /* no valid key for signing found, also bad */
+    GNUNET_break (0);
+    return 16;
+  }
+
+  /* check signing does not work if the key is unknown */
+  if (check_sigs)
+  {
+    struct TALER_PlanchetDetail pd;
+    struct TALER_CsPubHashP rnd;
+    struct TALER_CRYPTO_CsSignRequest csr;
+
+    GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_WEAK,
+                                &rnd,
+                                sizeof (rnd));
+    pd.blinded_planchet.cipher = TALER_DENOMINATION_CS;
+    GNUNET_assert (GNUNET_YES ==
+                   TALER_planchet_prepare (&keys[0].denom_pub,
+                                           &alg_values[0],
+                                           &bks[0],
+                                           &coin_priv[0],
+                                           NULL, /* no age commitment */
+                                           &c_hash[0],
+                                           &pd));
+    csr.h_cs = &rnd;
+    csr.blinded_planchet
+      = &pd.blinded_planchet.details.cs_blinded_planchet;
+    ec = TALER_CRYPTO_helper_cs_batch_sign (
+      dh,
+      &csr,
+      1,
+      false,
+      &ds[0]);
+    if (TALER_EC_EXCHANGE_GENERIC_DENOMINATION_KEY_UNKNOWN != ec)
+    {
+      if (TALER_EC_NONE == ec)
+        TALER_blinded_denom_sig_free (ds);
+      GNUNET_break (0);
+      return 17;
+    }
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Signing with invalid key %s failed as desired\n",
+                GNUNET_h2s (&rnd.hash));
+  }
+  return 0;
+}
+
+
 /**
  * Benchmark signing logic.
  *
@@ -618,18 +841,20 @@ perf_signing (struct TALER_CRYPTO_CsDenominationHelper *dh,
       {
         struct TALER_CoinPubHashP c_hash;
         struct TALER_PlanchetDetail pd;
+        struct TALER_CRYPTO_CsDeriveRequest cdr = {
+          .h_cs = &keys[i].h_cs,
+          .nonce = &pd.blinded_planchet.details.cs_blinded_planchet.nonce
+        };
 
         pd.blinded_planchet.cipher = TALER_DENOMINATION_CS;
         TALER_cs_withdraw_nonce_derive (&ps,
                                         &pd.blinded_planchet.details.
                                         cs_blinded_planchet.nonce);
         alg_values.cipher = TALER_DENOMINATION_CS;
-        ec = TALER_CRYPTO_helper_cs_r_derive_melt (
+        ec = TALER_CRYPTO_helper_cs_r_derive (
           dh,
-          &keys[i].h_cs,
-          &pd.blinded_planchet.
-          details.
-          cs_blinded_planchet.nonce,
+          &cdr,
+          true,
           &alg_values.details.cs_values);
         if (TALER_EC_NONE != ec)
           continue;
@@ -652,12 +877,15 @@ perf_signing (struct TALER_CRYPTO_CsDenominationHelper *dh,
         {
           struct GNUNET_TIME_Absolute start = GNUNET_TIME_absolute_get ();
           struct GNUNET_TIME_Relative delay;
+          struct TALER_CRYPTO_CsSignRequest csr;
 
-          ec = TALER_CRYPTO_helper_cs_sign_melt (
+          csr.h_cs = &keys[i].h_cs;
+          csr.blinded_planchet
+            = &pd.blinded_planchet.details.cs_blinded_planchet;
+          ec = TALER_CRYPTO_helper_cs_sign (
             dh,
-            &keys[i].h_cs,
-            &pd.blinded_planchet.details.
-            cs_blinded_planchet,
+            &csr,
+            true,
             &ds);
           if (TALER_EC_NONE != ec)
             break;
@@ -795,6 +1023,34 @@ run_test (void)
     ret = test_r_derive (dh);
   if (0 == ret)
     ret = test_signing (dh);
+  if (0 == ret)
+    ret = test_batch_signing (dh,
+                              2,
+                              true);
+  if (0 == ret)
+    ret = test_batch_signing (dh,
+                              256,
+                              true);
+  for (unsigned int i = 0; i<5; i++)
+  {
+    static unsigned int batches[] = { 1, 4, 16, 64, 256 };
+    unsigned int batch_size = batches[i];
+    struct GNUNET_TIME_Absolute start;
+    struct GNUNET_TIME_Relative duration;
+
+    start = GNUNET_TIME_absolute_get ();
+    if (0 != ret)
+      break;
+    ret = test_batch_signing (dh,
+                              batch_size,
+                              false);
+    duration = GNUNET_TIME_absolute_get_duration (start);
+    fprintf (stderr,
+             "%4u (batch) signature operations took %s (total real time)\n",
+             (unsigned int) batch_size,
+             GNUNET_STRINGS_relative_time_to_string (duration,
+                                                     GNUNET_YES));
+  }
   if (0 == ret)
     ret = perf_signing (dh,
                         "sequential");
@@ -818,13 +1074,14 @@ main (int argc,
   int ret;
   enum GNUNET_OS_ProcessStatusType type;
   unsigned long code;
+  const char *loglev = "WARNING";
 
   (void) argc;
   (void) argv;
   unsetenv ("XDG_DATA_HOME");
   unsetenv ("XDG_CONFIG_HOME");
   GNUNET_log_setup ("test-helper-cs",
-                    "WARNING",
+                    loglev,
                     NULL);
   GNUNET_OS_init (TALER_project_data_default ());
   libexec_dir = GNUNET_OS_installation_get_path (GNUNET_OS_IPK_BINDIR);
@@ -840,7 +1097,7 @@ main (int argc,
                                     "-c",
                                     "test_helper_cs.conf",
                                     "-L",
-                                    "WARNING",
+                                    loglev,
                                     NULL);
   if (NULL == helper)
   {
