@@ -27,8 +27,8 @@
  * be urlencoded.
  *
  * @param c character to look at
- * @return #GNUNET_YES if @a c needs to be urlencoded,
- *         #GNUNET_NO otherwise
+ * @return true if @a c needs to be urlencoded,
+ *         false otherwise
  */
 static bool
 is_reserved (char c)
@@ -48,11 +48,11 @@ is_reserved (char c)
   case 'P': case 'Q': case 'R': case 'S': case 'T':
   case 'U': case 'V': case 'W': case 'X': case 'Y': case 'Z':
   case '-': case '.': case '_': case '~':
-    return GNUNET_NO;
+    return false;
   default:
     break;
   }
-  return GNUNET_YES;
+  return true;
 }
 
 
@@ -68,7 +68,7 @@ urlencode_len (const char *s)
 {
   size_t len = 0;
   for (; *s != '\0'; len++, s++)
-    if (GNUNET_YES == is_reserved (*s))
+    if (is_reserved (*s))
       len += 2;
   return len;
 }
@@ -102,12 +102,6 @@ buffer_write_urlencode (struct GNUNET_Buffer *buf,
 }
 
 
-/**
- * URL-encode a string according to rfc3986.
- *
- * @param s string to encode
- * @returns the urlencoded string, the caller must free it with #GNUNET_free()
- */
 char *
 TALER_urlencode (const char *s)
 {
@@ -212,20 +206,6 @@ serialize_arguments (struct GNUNET_Buffer *buf,
 }
 
 
-/**
- * Make an absolute URL with query parameters.
- *
- * If a 'value' is given as NULL, both the key and the value are skipped. Note
- * that a NULL value does not terminate the list, only a NULL key signals the
- * end of the list of arguments.
- *
- * @param base_url absolute base URL to use
- * @param path path of the url
- * @param ... NULL-terminated key-value pairs (char *) for query parameters,
- *        the value will be url-encoded
- * @returns the URL (must be freed with #GNUNET_free) or
- *          NULL if an error occurred.
- */
 char *
 TALER_url_join (const char *base_url,
                 const char *path,
@@ -240,15 +220,17 @@ TALER_url_join (const char *base_url,
   if (0 == strlen (base_url))
   {
     /* base URL can't be empty */
-    GNUNET_break (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "Empty base URL specified\n");
     return NULL;
   }
   if ('/' != base_url[strlen (base_url) - 1])
   {
     /* Must be an actual base URL! */
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Base URL `%s' does not end with '/'\n",
-                base_url);
+                "Base URL `%s' does not end with '/', cannot join with `%s'\n",
+                base_url,
+                path);
     return NULL;
   }
   if ('/' == path[0])
@@ -280,21 +262,6 @@ TALER_url_join (const char *base_url,
 }
 
 
-/**
- * Make an absolute URL for the given parameters.
- *
- * If a 'value' is given as NULL, both the key and the value are skipped. Note
- * that a NULL value does not terminate the list, only a NULL key signals the
- * end of the list of arguments.
- *
- * @param proto protocol for the URL (typically https)
- * @param host hostname for the URL
- * @param prefix prefix for the URL
- * @param path path for the URL
- * @param args NULL-terminated key-value pairs (char *) for query parameters,
- *        the value will be url-encoded
- * @returns the URL, must be freed with #GNUNET_free
- */
 char *
 TALER_url_absolute_raw_va (const char *proto,
                            const char *host,
@@ -327,21 +294,6 @@ TALER_url_absolute_raw_va (const char *proto,
 }
 
 
-/**
- * Make an absolute URL for the given parameters.
- *
- * If a 'value' is given as NULL, both the key and the value are skipped. Note
- * that a NULL value does not terminate the list, only a NULL key signals the
- * end of the list of arguments.
- *
- * @param proto protocol for the URL (typically https)
- * @param host hostname for the URL
- * @param prefix prefix for the URL
- * @param path path for the URL
- * @param ... NULL-terminated key-value pairs (char *) for query parameters,
- *        the value will be url-encoded
- * @return the URL, must be freed with #GNUNET_free
- */
 char *
 TALER_url_absolute_raw (const char *proto,
                         const char *host,
@@ -364,4 +316,20 @@ TALER_url_absolute_raw (const char *proto,
 }
 
 
+bool
+TALER_url_valid_charset (const char *url)
+{
+  for (unsigned int i = 0; '\0' != url[i]; i++)
+  {
+#define ALLOWED_CHARACTERS \
+  "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/:;&?-.,=_~%+"
+    if (NULL == strchr (ALLOWED_CHARACTERS,
+                        (int) url[i]))
+      return false;
+#undef ALLOWED_CHARACTERS
+  }
+  return true;
+}
+
+
 /* end of url.c */
