@@ -28,7 +28,7 @@ const char *
 TALER_b2s (const void *buf,
            size_t buf_size)
 {
-  static GNUNET_THREAD_LOCAL char ret[9];
+  static TALER_THREAD_LOCAL char ret[9];
   struct GNUNET_HashCode hc;
   char *tmp;
 
